@@ -17,6 +17,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 #include "config.h"
+
 #include "episodes.h"
 #include "file.h"
 #include "joystick.h"
@@ -30,9 +31,15 @@
 #include "video.h"
 #include "video_scale.h"
 
-#include <unistd.h>
+#include <stdio.h>
 #include <sys/stat.h>
 
+#ifdef _MSC_VER
+#include <direct.h>
+#define mkdir _mkdir
+#else
+#include <unistd.h>
+#endif
 
 /* Configuration Load/Save handler */
 
@@ -41,10 +48,33 @@ const JE_byte cryptKey[10] = /* [1..10] */
 	15, 50, 89, 240, 147, 34, 86, 9, 32, 208
 };
 
-const JE_KeySettingType defaultKeySettings =
+const DosKeySettings defaultDosKeySettings =
 {
-	SDLK_UP, SDLK_DOWN, SDLK_LEFT, SDLK_RIGHT, SDLK_SPACE, SDLK_RETURN, SDLK_LCTRL, SDLK_LALT
-/*	72, 80, 75, 77, 57, 28, 29, 56*/
+	72, 80, 75, 77, 57, 28, 29, 56
+};
+
+const KeySettings defaultKeySettings =
+{
+	SDL_SCANCODE_UP,
+	SDL_SCANCODE_DOWN,
+	SDL_SCANCODE_LEFT,
+	SDL_SCANCODE_RIGHT,
+	SDL_SCANCODE_SPACE,
+	SDL_SCANCODE_RETURN,
+	SDL_SCANCODE_LCTRL,
+	SDL_SCANCODE_LALT,
+};
+
+static const char *const keySettingNames[] =
+{
+	"up",
+	"down",
+	"left",
+	"right",
+	"fire",
+	"change fire",
+	"left sidekick",
+	"right sidekick",
 };
 
 const char defaultHighScoreNames[34][23] = /* [1..34] of string [22] */
@@ -163,7 +193,8 @@ char    lastLevelName[11], levelName[11]; /* string [10] */
 JE_byte mainLevel, nextLevel, saveLevel;   /*Current Level #*/
 
 /* Keyboard Junk */
-JE_KeySettingType keySettings;
+DosKeySettings dosKeySettings;
+KeySettings keySettings;
 
 /* Configuration */
 JE_shortint levelFilter, levelFilterNew, levelBrightness, levelBrightnessChg;
@@ -213,96 +244,108 @@ JE_SaveGameTemp saveTemp;
 
 JE_word editorLevel;   /*Initial value 800*/
 
-
-cJSON *load_json( const char *filename )
-{
-	FILE *f = dir_fopen_warn(get_user_directory(), filename, "rb");
-	if (f == NULL)
-		return NULL;
-	
-	size_t buffer_len = ftell_eof(f);
-	char *buffer = malloc(buffer_len + 1);
-	
-	fread(buffer, 1, buffer_len, f);
-	buffer[buffer_len] = '\0';
-	
-	fclose(f);
-	
-	cJSON *root = cJSON_Parse(buffer);
-	
-	free(buffer);
-	
-	return root;
-}
-
-void save_json( cJSON *root, const char *filename )
-{
-	FILE *f = dir_fopen_warn(get_user_directory(), filename, "w+");
-	if (f == NULL)
-		return;
-	
-	char *buffer = cJSON_Print(root);
-	
-	if (buffer != NULL)
-	{
-		fputs(buffer, f);
-		free(buffer);
-	}
-	
-	fclose(f);
-}
+Config opentyrian_config;  // implicitly initialized
 
 bool load_opentyrian_config( void )
 {
 	// defaults
-	fullscreen_enabled = false;
+	fullscreen_display = -1;
 	set_scaler_by_name("Scale2x");
+	memcpy(keySettings, defaultKeySettings, sizeof(keySettings));
 	
-	cJSON *root = load_json("opentyrian.conf");
-	if (root == NULL)
+	Config *config = &opentyrian_config;
+	
+	FILE *file = dir_fopen_warn(get_user_directory(), "opentyrian.cfg", "r");
+	if (file == NULL)
 		return false;
 	
-	cJSON *section = cJSON_GetObjectItem(root, "video");
-	if (section != NULL)
+	if (!config_parse(config, file))
 	{
-		cJSON *setting;
+		fclose(file);
 		
-		if ((setting = cJSON_GetObjectItem(section, "fullscreen")))
-			fullscreen_enabled = (setting->type == cJSON_True);
-		
-		if ((setting = cJSON_GetObjectItem(section, "scaler")))
-			set_scaler_by_name(setting->valuestring);
+		return false;
 	}
 	
-	cJSON_Delete(root);
+	ConfigSection *section;
+	
+	section = config_find_section(config, "video", NULL);
+	if (section != NULL)
+	{
+		config_get_int_option(section, "fullscreen", &fullscreen_display);
+		
+		const char *scaler;
+		if (config_get_string_option(section, "scaler", &scaler))
+			set_scaler_by_name(scaler);
+		
+		const char *scaling_mode;
+		if (config_get_string_option(section, "scaling_mode", &scaling_mode))
+			set_scaling_mode_by_name(scaling_mode);
+	}
+
+	section = config_find_section(config, "keyboard", NULL);
+	if (section != NULL)
+	{
+		for (size_t i = 0; i < COUNTOF(keySettings); ++i)
+		{
+			const char *keyName;
+			if (config_get_string_option(section, keySettingNames[i], &keyName))
+			{
+				SDL_Scancode scancode = SDL_GetScancodeFromName(keyName);
+				if (scancode != SDL_SCANCODE_UNKNOWN)
+					keySettings[i] = scancode;
+			}
+		}
+	}
+
+	fclose(file);
 	
 	return true;
 }
 
 bool save_opentyrian_config( void )
 {
-	cJSON *root = load_json("opentyrian.conf");
-	if (root == NULL)
-		root = cJSON_CreateObject();
+	Config *config = &opentyrian_config;
 	
-	cJSON *section;
+	ConfigSection *section;
 	
-	section = cJSON_CreateOrGetObjectItem(root, "video");
-	cJSON_ForceType(section, cJSON_Object);
+	section = config_find_or_add_section(config, "video", NULL);
+	if (section == NULL)
+		exit(EXIT_FAILURE);  // out of memory
 	
+	config_set_int_option(section, "fullscreen", fullscreen_display);
+	
+	config_set_string_option(section, "scaler", scalers[scaler].name);
+	
+	config_set_string_option(section, "scaling_mode", scaling_mode_names[scaling_mode]);
+
+	section = config_find_or_add_section(config, "keyboard", NULL);
+	if (section == NULL)
+		exit(EXIT_FAILURE);  // out of memory
+
+	for (size_t i = 0; i < COUNTOF(keySettings); ++i)
 	{
-		cJSON *setting;
-		
-		setting = cJSON_CreateOrGetObjectItem(section, "fullscreen");
-		cJSON_SetBoolean(setting, fullscreen_enabled);
-		
-		setting = cJSON_CreateOrGetObjectItem(section, "scaler");
-		cJSON_SetString(setting, scalers[scaler].name);
+		const char *keyName = SDL_GetScancodeName(keySettings[i]);
+		if (keyName[0] == '\0')
+			keyName = NULL;
+		config_set_string_option(section, keySettingNames[i], keyName);
 	}
+
+#ifndef TARGET_WIN32
+	mkdir(get_user_directory(), 0700);
+#else
+	mkdir(get_user_directory());
+#endif
 	
-	save_json(root, "opentyrian.conf");
+	FILE *file = dir_fopen(get_user_directory(), "opentyrian.cfg", "w");
+	if (file == NULL)
+		return false;
 	
-	cJSON_Delete(root);
+	config_write(config, file);
+	
+#if _POSIX_C_SOURCE >= 1 || _XOPEN_SOURCE || _POSIX_SOURCE
+	fsync(fileno(file));
+#endif
+	fclose(file);
 	
 	return true;
 }
@@ -701,24 +744,37 @@ void JE_decryptSaveTemp( void )
 	memcpy(&saveTemp, &s2, sizeof(s2));
 }
 
-#ifndef TARGET_MACOSX
 const char *get_user_directory( void )
 {
 	static char user_dir[500] = "";
 	
 	if (strlen(user_dir) == 0)
 	{
-#ifdef TARGET_UNIX
-		if (getenv("HOME"))
-			snprintf(user_dir, sizeof(user_dir), "%s/.opentyrian", getenv("HOME"));
+#ifndef TARGET_WIN32
+		char *xdg_config_home = getenv("XDG_CONFIG_HOME");
+		if (xdg_config_home != NULL)
+		{
+			snprintf(user_dir, sizeof(user_dir), "%s/opentyrian", xdg_config_home);
+		}
+		else
+		{
+			char *home = getenv("HOME");
+			if (home != NULL)
+			{
+				snprintf(user_dir, sizeof(user_dir), "%s/.config/opentyrian", home);
+			}
+			else
+			{
+				strcpy(user_dir, ".");
+			}
+		}
 #else
 		strcpy(user_dir, ".");
-#endif // TARGET_UNIX
+#endif
 	}
 	
 	return user_dir;
 }
-#endif // TARGET_MACOSX
 
 // for compatibility
 Uint8 joyButtonAssign[4] = {1, 4, 5, 5};
@@ -732,33 +788,31 @@ void JE_loadConfiguration( void )
 	int y;
 	
 	fi = dir_fopen_warn(get_user_directory(), "tyrian.cfg", "rb");
-	if (fi && ftell_eof(fi) == 20 + sizeof(keySettings))
+	if (fi && ftell_eof(fi) == 28)
 	{
-		/* SYN: I've hardcoded the sizes here because the .CFG file format is fixed
-		   anyways, so it's not like they'll change. */
 		background2 = 0;
-		efread(&background2, 1, 1, fi);
-		efread(&gameSpeed, 1, 1, fi);
+		fread_bool_die(&background2, fi);
+		fread_u8_die(&gameSpeed, 1, fi);
 		
-		efread(&inputDevice_, 1, 1, fi);
-		efread(&jConfigure, 1, 1, fi);
+		fread_u8_die(&inputDevice_, 1, fi);
+		fread_u8_die(&jConfigure, 1, fi);
 		
-		efread(&versionNum, 1, 1, fi);
+		fread_u8_die(&versionNum, 1, fi);
 		
-		efread(&processorType, 1, 1, fi);
-		efread(&midiPort, 1, 1, fi);
-		efread(&soundEffects, 1, 1, fi);
-		efread(&gammaCorrection, 1, 1, fi);
-		efread(&difficultyLevel, 1, 1, fi);
+		fread_u8_die(&processorType, 1, fi);
+		fread_u8_die(&midiPort, 1, fi);
+		fread_u8_die(&soundEffects, 1, fi);
+		fread_u8_die(&gammaCorrection, 1, fi);
+		fread_s8_die(&difficultyLevel, 1, fi);
 		
-		efread(joyButtonAssign, 1, 4, fi);
+		fread_u8_die(joyButtonAssign, 4, fi);
 		
-		efread(&tyrMusicVolume, 2, 1, fi);
-		efread(&fxVolume, 2, 1, fi);
+		fread_u16_die(&tyrMusicVolume, 1, fi);
+		fread_u16_die(&fxVolume, 1, fi);
 		
-		efread(inputDevice, 1, 2, fi);
-		
-		efread(keySettings, sizeof(*keySettings), COUNTOF(keySettings), fi);
+		fread_u8_die(inputDevice, 2, fi);
+
+		fread_u8_die(dosKeySettings, 8, fi);
 		
 		fclose(fi);
 	}
@@ -767,7 +821,7 @@ void JE_loadConfiguration( void )
 		printf("\nInvalid or missing TYRIAN.CFG! Continuing using defaults.\n\n");
 		
 		soundEffects = 1;
-		memcpy(&keySettings, &defaultKeySettings, sizeof(keySettings));
+		memcpy(&dosKeySettings, &defaultDosKeySettings, sizeof(dosKeySettings));
 		background2 = true;
 		tyrMusicVolume = fxVolume = 128;
 		gammaCorrection = 0;
@@ -791,7 +845,7 @@ void JE_loadConfiguration( void )
 	{
 
 		fseek(fi, 0, SEEK_SET);
-		efread(saveTemp, 1, sizeof(saveTemp), fi);
+		fread_die(saveTemp, 1, sizeof(saveTemp), fi);
 		JE_decryptSaveTemp();
 
 		/* SYN: The original mostly blasted the save file into raw memory. However, our lives are not so
@@ -854,7 +908,7 @@ void JE_loadConfiguration( void )
 		}
 
 		/* SYN: This is truncating to bytes. I have no idea what this is doing or why. */
-		/* TODO: Figure out what this is about and make sure it isn't broked. */
+		/* TODO: Figure out what this is about and make sure it isn't broken. */
 		editorLevel = (saveTemp[SIZEOF_SAVEGAMETEMP - 5] << 8) | saveTemp[SIZEOF_SAVEGAMETEMP - 6];
 
 		fclose(fi);
@@ -894,15 +948,6 @@ void JE_loadConfiguration( void )
 
 void JE_saveConfiguration( void )
 {
-#ifdef TARGET_UNIX
-	if (getenv("HOME"))
-	{
-		char dir[1000];
-		snprintf(dir, sizeof(dir), "%s/.opentyrian", getenv("HOME"));
-		mkdir(dir, 0755);
-	}
-#endif /* HOME */
-	
 	FILE *f;
 	JE_byte *p;
 	int z;
@@ -971,48 +1016,55 @@ void JE_saveConfiguration( void )
 	
 	JE_encryptSaveTemp();
 	
-	f = dir_fopen_warn(get_user_directory(), "tyrian.sav", "wb");
-	if (f)
-	{
-		efwrite(saveTemp, 1, sizeof(saveTemp), f);
-		fclose(f);
-#if (_BSD_SOURCE || _XOPEN_SOURCE >= 500)
-		sync();
+#ifndef TARGET_WIN32
+	mkdir(get_user_directory(), 0700);
+#else
+	mkdir(get_user_directory());
 #endif
+	
+	f = dir_fopen_warn(get_user_directory(), "tyrian.sav", "wb");
+	if (f != NULL)
+	{
+		fwrite_die(saveTemp, 1, sizeof(saveTemp), f);
+
+#if _POSIX_C_SOURCE >= 1 || _XOPEN_SOURCE || _POSIX_SOURCE
+		fsync(fileno(f));
+#endif
+		fclose(f);
 	}
+	
 	JE_decryptSaveTemp();
 	
 	f = dir_fopen_warn(get_user_directory(), "tyrian.cfg", "wb");
-	if (f)
+	if (f != NULL)
 	{
-		efwrite(&background2, 1, 1, f);
-		efwrite(&gameSpeed, 1, 1, f);
+		fwrite_bool_die(&background2, f);
+		fwrite_u8_die(&gameSpeed, 1, f);
 		
-		efwrite(&inputDevice_, 1, 1, f);
-		efwrite(&jConfigure, 1, 1, f);
+		fwrite_u8_die(&inputDevice_, 1, f);
+		fwrite_u8_die(&jConfigure, 1, f);
 		
-		efwrite(&versionNum, 1, 1, f);
-		efwrite(&processorType, 1, 1, f);
-		efwrite(&midiPort, 1, 1, f);
-		efwrite(&soundEffects, 1, 1, f);
-		efwrite(&gammaCorrection, 1, 1, f);
-		efwrite(&difficultyLevel, 1, 1, f);
-		efwrite(joyButtonAssign, 1, 4, f);
+		fwrite_u8_die(&versionNum, 1, f);
+		fwrite_u8_die(&processorType, 1, f);
+		fwrite_u8_die(&midiPort, 1, f);
+		fwrite_u8_die(&soundEffects, 1, f);
+		fwrite_u8_die(&gammaCorrection, 1, f);
+		fwrite_s8_die(&difficultyLevel, 1, f);
+		fwrite_u8_die(joyButtonAssign, 4, f);
 		
-		efwrite(&tyrMusicVolume, 2, 1, f);
-		efwrite(&fxVolume, 2, 1, f);
+		fwrite_u16_die(&tyrMusicVolume, f);
+		fwrite_u16_die(&fxVolume, f);
 		
-		efwrite(inputDevice, 1, 2, f);
+		fwrite_u8_die(inputDevice, 2, f);
 		
-		efwrite(keySettings, sizeof(*keySettings), COUNTOF(keySettings), f);
+		fwrite_u8_die(dosKeySettings, 8, f);
 		
+#if _POSIX_C_SOURCE >= 1 || _XOPEN_SOURCE || _POSIX_SOURCE
+		fsync(fileno(f));
+#endif
 		fclose(f);
 	}
 	
 	save_opentyrian_config();
-	
-#if (_BSD_SOURCE || _XOPEN_SOURCE >= 500)
-	sync();
-#endif
 }
 
