@@ -32,7 +32,7 @@
  * to not have available.  Destruct has no configuration options in game, but
  * that doesn't stop us from changing various limiting vars and letting
  * people remap the keyboard.  AIs may also be introduced here; fighting a
- * stateless AI isn't really challenging afterall.
+ * stateless AI isn't really challenging after all.
  *
  * This hidden config also allows for a hidden game mode!  Though as a custom
  * game mode wouldn't show up in the data files it forces us to distinguish
@@ -40,21 +40,22 @@
  * is only used with loaded data.
  *
  * Things I wanted to do but can't: Remove references to VGAScreen.  For
- * a multitude of reasons this just isn't feasable.  It would have been nice
+ * a multitude of reasons this just isn't feasible.  It would have been nice
  * to increase the playing field though...
  */
 
 /*** Headers ***/
-#include "opentyr.h"
 #include "destruct.h"
 
 #include "config.h"
+#include "config_file.h"
 #include "fonthand.h"
 #include "helptext.h"
 #include "keyboard.h"
 #include "loudness.h"
 #include "mtrand.h"
 #include "nortsong.h"
+#include "opentyr.h"
 #include "palette.h"
 #include "picload.h"
 #include "sprite.h"
@@ -163,7 +164,7 @@ struct destruct_moves_s {
 	bool actions[MAX_MOVE];
 };
 struct destruct_keys_s {
-	SDLKey Config[MAX_KEY][MAX_KEY_OPTIONS];
+	SDL_Scancode Config[MAX_KEY][MAX_KEY_OPTIONS];
 };
 struct destruct_ai_s {
 
@@ -323,9 +324,9 @@ static JE_byte basetypes[10][11] /*[1..8, 1..11]*/ = /* [0] is amount of units*/
 	{8, UNIT_HELI, UNIT_HELI, UNIT_HELI, UNIT_HELI,      UNIT_HELI,   UNIT_HELI,      UNIT_HELI,   UNIT_HELI,   UNIT_HELI,   UNIT_HELI},   /*Strong Heli attack fleet*/
 	{4, UNIT_TANK, UNIT_TANK, UNIT_TANK, UNIT_TANK,      UNIT_NUKE,   UNIT_NUKE,      UNIT_DIRT,   UNIT_MAGNET, UNIT_JUMPER, UNIT_JUMPER}, /*Weak   Heli defense fleet*/
 	{8, UNIT_TANK, UNIT_NUKE, UNIT_DIRT, UNIT_SATELLITE, UNIT_MAGNET, UNIT_LASER,     UNIT_JUMPER, UNIT_HELI,   UNIT_TANK,   UNIT_NUKE},   /*Overpowering fleet*/
-	{4, UNIT_TANK, UNIT_TANK, UNIT_NUKE, UNIT_DIRT,      UNIT_TANK,   UNIT_LASER,     UNIT_JUMPER, UNIT_HELI,   UNIT_NUKE,   UNIT_JUMPER},  /*Weak fleet*/
-	{1, UNIT_TANK, UNIT_TANK, UNIT_TANK, UNIT_TANK,      UNIT_TANK,   UNIT_TANK,      UNIT_TANK,   UNIT_TANK,   UNIT_TANK,   UNIT_TANK},   /*Custom1, to be edited*/
-	{1, UNIT_TANK, UNIT_TANK, UNIT_TANK, UNIT_TANK,      UNIT_TANK,   UNIT_TANK,      UNIT_TANK,   UNIT_TANK,   UNIT_TANK,   UNIT_TANK}   /*Custom2, to be edited*/
+	{4, UNIT_TANK, UNIT_TANK, UNIT_NUKE, UNIT_DIRT,      UNIT_TANK,   UNIT_LASER,     UNIT_JUMPER, UNIT_HELI,   UNIT_NUKE,   UNIT_JUMPER}, /*Weak fleet*/
+	{5, UNIT_TANK, UNIT_TANK, UNIT_NUKE, UNIT_DIRT,      UNIT_DIRT,   UNIT_SATELLITE, UNIT_MAGNET, UNIT_LASER,  UNIT_JUMPER, UNIT_HELI},   /*Left custom*/
+	{5, UNIT_TANK, UNIT_TANK, UNIT_NUKE, UNIT_DIRT,      UNIT_DIRT,   UNIT_SATELLITE, UNIT_MAGNET, UNIT_LASER,  UNIT_JUMPER, UNIT_HELI},   /*Right custom*/
 };
 static const unsigned int baseLookup[MAX_PLAYERS][MAX_MODES] =
 {
@@ -346,25 +347,25 @@ static const JE_byte ModeScore[MAX_PLAYERS][MAX_MODES] =
 	{1, 0, 5, 0, 1, 1}
 };
 
-static SDLKey defaultKeyConfig[MAX_PLAYERS][MAX_KEY][MAX_KEY_OPTIONS] =
+static SDL_Scancode defaultKeyConfig[MAX_PLAYERS][MAX_KEY][MAX_KEY_OPTIONS] =
 {
-	{	{SDLK_c},
-		{SDLK_v},
-		{SDLK_a},
-		{SDLK_z},
-		{SDLK_LALT},
-		{SDLK_x, SDLK_LSHIFT},
-		{SDLK_LCTRL},
-		{SDLK_SPACE}
+	{	{SDL_SCANCODE_C},
+		{SDL_SCANCODE_V},
+		{SDL_SCANCODE_A},
+		{SDL_SCANCODE_Z},
+		{SDL_SCANCODE_LALT},
+		{SDL_SCANCODE_X, SDL_SCANCODE_LSHIFT},
+		{SDL_SCANCODE_LCTRL},
+		{SDL_SCANCODE_SPACE}
 	},
-	{	{SDLK_LEFT, SDLK_KP4},
-		{SDLK_RIGHT, SDLK_KP6},
-		{SDLK_UP, SDLK_KP8},
-		{SDLK_DOWN, SDLK_KP2},
-		{SDLK_BACKSLASH, SDLK_KP5},
-		{SDLK_INSERT, SDLK_RETURN, SDLK_KP0, SDLK_KP_ENTER},
-		{SDLK_PAGEUP, SDLK_KP9},
-		{SDLK_PAGEDOWN, SDLK_KP3}
+	{	{SDL_SCANCODE_LEFT, SDL_SCANCODE_KP_4},
+		{SDL_SCANCODE_RIGHT, SDL_SCANCODE_KP_6},
+		{SDL_SCANCODE_UP, SDL_SCANCODE_KP_8},
+		{SDL_SCANCODE_DOWN, SDL_SCANCODE_KP_2},
+		{SDL_SCANCODE_BACKSLASH, SDL_SCANCODE_KP_5},
+		{SDL_SCANCODE_INSERT, SDL_SCANCODE_RETURN, SDL_SCANCODE_KP_0, SDL_SCANCODE_KP_ENTER},
+		{SDL_SCANCODE_PAGEUP, SDL_SCANCODE_KP_9},
+		{SDL_SCANCODE_PAGEDOWN, SDL_SCANCODE_KP_3}
 	}
 };
 
@@ -380,307 +381,156 @@ static struct destruct_shot_s   * shotRec;
 static struct destruct_explo_s  * exploRec;
 
 
-/*** Startup ***/
-static enum de_unit_t string_to_unit_enum(const char * str) {
+static const char *player_names[] =
+{
+	"left", "right",
+};
 
-	// A config helper function.  Probably not useful anywhere else.
-	enum de_unit_t i;
-	static const char * unit_names[] =
-	{ "UNIT_TANK", "UNIT_NUKE", "UNIT_DIRT", "UNIT_SATELLITE",
-      "UNIT_MAGNET", "UNIT_LASER", "UNIT_JUMPER", "UNIT_HELI" };
+static const char *key_names[] =
+{
+	"left", "right", "up", "down",
+	"change", "fire", "previous weapon", "next weapon",
+};
 
-	for (i = UNIT_FIRST; i < MAX_UNITS; i++) {
-		if(strcmp(unit_names[i], str) == 0) { return(i); }
-	}
+static const char *unit_names[] =
+{
+	"tank", "nuke", "dirt", "satellite",
+	"magnet", "laser", "jumper", "heli",
+};
 
-    return(UNIT_NONE);
+static enum de_unit_t get_unit_by_name( const char *unit_name )
+{
+	for (enum de_unit_t unit = UNIT_FIRST; unit < MAX_UNITS; ++unit)
+		if (strcmp(unit_name, unit_names[unit]) == 0)
+			return unit;
+	
+	return UNIT_NONE;
 }
-static bool write_default_destruct_config( void ) {
 
-	cJSON * root;
-	cJSON * level1, * level2, * level3, * setting;
-
-
-	//If I read the file right, all of these will return NULL on failure.
-	//Well that'll be a little bit tedious to check for each time, but using
-	//gotos can help clear everything up since only one thing is freed.
-	if((root = cJSON_CreateObject()) == NULL) { goto label_failure; }
-
-
-	if((level1 = cJSON_CreateOrGetObjectItem(root, "general")) == NULL) { goto label_failure; }
-	cJSON_ForceType(level1, cJSON_Object);
-
-	//general
-	if((setting = cJSON_CreateOrGetObjectItem(level1, "alwaysalias")) == NULL) { goto label_failure; }
-	cJSON_SetBoolean(setting, false);
-	if((setting = cJSON_CreateOrGetObjectItem(level1, "tracerlaser")) == NULL) { goto label_failure; }
-	cJSON_SetBoolean(setting, false);
-	if((setting = cJSON_CreateOrGetObjectItem(level1, "max_shots")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, 40);
-	if((setting = cJSON_CreateOrGetObjectItem(level1, "min_walls")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, 20);
-	if((setting = cJSON_CreateOrGetObjectItem(level1, "max_walls")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, 20);
-	if((setting = cJSON_CreateOrGetObjectItem(level1, "max_explosions")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, 40);
-
-	//players general
-	if((level2 = cJSON_CreateOrGetObjectItem(level1, "player1")) == NULL) { goto label_failure; }
-	cJSON_ForceType(level2, cJSON_Object);
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "ai")) == NULL) { goto label_failure; }
-	cJSON_SetBoolean(setting, true);
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "jumper_fires_straight")) == NULL) { goto label_failure; }
-	cJSON_SetBoolean(setting, true);
-
-	if((level3 = cJSON_CreateOrGetObjectItem(level2, "keys")) == NULL) { goto label_failure; }
-	cJSON_ForceType(level3, cJSON_Object);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "__comment")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "You may configure the keys here.  Nums correspond to SDL defines.  It's better than nothing.");
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "left1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_c);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "right1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_v);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "up1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_a);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "down1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_z);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "change1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_LALT);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "fire1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_x);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "fire2")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_LSHIFT);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "cyup1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_LCTRL);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "cydn1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_SPACE);
-
-	if((level2 = cJSON_CreateOrGetObjectItem(level1, "player2")) == NULL) { goto label_failure; }
-	cJSON_ForceType(level2, cJSON_Object);
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "ai")) == NULL) { goto label_failure; }
-	cJSON_SetBoolean(setting, false);
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "jumper_fires_straight")) == NULL) { goto label_failure; }
-	cJSON_SetBoolean(setting, false);
-
-	if((level3 = cJSON_CreateOrGetObjectItem(level2, "keys")) == NULL) { goto label_failure; }
-	cJSON_ForceType(level3, cJSON_Object);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "left1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_LEFT);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "left2")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_KP4);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "right1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_RIGHT);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "right2")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_KP6);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "up1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_UP);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "up2")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_KP8);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "down1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_DOWN);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "down2")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_KP2);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "change1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_BACKSLASH);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "change2")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_KP5);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "fire1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_INSERT);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "fire2")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_RETURN);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "fire3")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_KP0);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "fire4")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_KP_ENTER);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "cyup1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_PAGEUP);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "cyup2")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_KP9);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "cydn1")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_PAGEDOWN);
-	if((setting = cJSON_CreateOrGetObjectItem(level3, "cydn2")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, SDLK_KP3);
-
-	//custom mode
-	if((level1 = cJSON_CreateOrGetObjectItem(root, "custom")) == NULL) { goto label_failure; }
-	cJSON_ForceType(level1, cJSON_Object);
-
-	if((setting = cJSON_CreateOrGetObjectItem(level1, "enable")) == NULL) { goto label_failure; }
-	cJSON_SetBoolean(setting, false);
-
-	//player 1 (I could but won't bother looping this)
-	if((level2 = cJSON_CreateOrGetObjectItem(level1, "player1")) == NULL) { goto label_failure; }
-	cJSON_ForceType(level2, cJSON_Object);
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "num_units")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, 10);
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "__comment")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "This handles probability.  Always have 10 entries.");
-
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit1")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_TANK");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit2")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_TANK");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit3")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_NUKE");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit4")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_DIRT");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit5")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_DIRT");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit6")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_SATELLITE");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit7")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_MAGNET");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit8")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_LASER");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit9")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_JUMPER");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit10")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_HELI");
-
-	if((level2 = cJSON_CreateOrGetObjectItem(level1, "player2")) == NULL) { goto label_failure; }
-	cJSON_ForceType(level2, cJSON_Object);
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "num_units")) == NULL) { goto label_failure; }
-	cJSON_SetNumber(setting, 10);
-
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit1")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_TANK");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit2")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_TANK");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit3")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_NUKE");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit4")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_DIRT");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit5")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_DIRT");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit6")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_SATELLITE");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit7")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_MAGNET");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit8")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_LASER");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit9")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_JUMPER");
-	if((setting = cJSON_CreateOrGetObjectItem(level2, "unit10")) == NULL) { goto label_failure; }
-	cJSON_SetString(setting, "UNIT_HELI");
-
-	save_json(root, "destruct.conf");
-	return(true);
-
-label_failure:
-	cJSON_Delete(root);
-	return(false);
-}
-static void load_destruct_config( void ) {
-
-	unsigned int j, k;
-	enum de_player_t i;
-	enum de_unit_t temp;
-	char buffer[40];
-	const char * key_names[] = { "left", "right", "up", "down", "change", "fire", "cyup", "cydn" };
-	cJSON * root;
-	cJSON * level1, * level2, * level3, * setting;
-
-	// The config file is not modified in game in order to 'keep' with the
-	// original (unconfigurable) feel.  This code was copied from elsewhere.
-	root = load_json("destruct.conf");
-	if (root == NULL) {
-		write_default_destruct_config();
-		return;
-	}
-
-	//load these general config items.  I don't consider sanity checks
-	//necessary; either the game isn't playable or you eat up all your memory
-	//when using unreasonable values.  Either way, no exploit here.
-	level1 = cJSON_GetObjectItem(root, "general");
-	if (level1 != NULL)
+static void load_destruct_config( Config *config_ )
+{
+	ConfigSection *section;
+	
+	section = config_find_or_add_section(config_, "destruct", NULL);
+	if (section == NULL)
+		exit(EXIT_FAILURE);  // out of memory
+	
+	config.alwaysalias = config_get_or_set_bool_option(section, "antialias craters", false, NO_YES);
+	
+	weaponSystems[UNIT_LASER][SHOT_LASERTRACER] = config_get_or_set_bool_option(section, "tracer laser", false, OFF_ON);
+	
+	config.max_shots = config_get_or_set_int_option(section, "max shots", 40);
+	config.max_explosions = config_get_or_set_int_option(section, "max explosions", 40);
+	config.min_walls = config_get_or_set_int_option(section, "min walls", 20);
+	config.max_walls = config_get_or_set_int_option(section, "max walls", 20);
+	
+	config.ai[0] = config_get_or_set_bool_option(section, "left ai", true, NO_YES);
+	config.jumper_straight[0] = config_get_or_set_bool_option(section, "left jumper fires straight", true, NO_YES);
+	config.ai[1] = config_get_or_set_bool_option(section, "right ai", false, NO_YES);
+	config.jumper_straight[1] = config_get_or_set_bool_option(section, "right jumper fires straight", false, NO_YES);
+	
+	// keyboard controls
+	
+	for (int p = 0; p < MAX_PLAYERS; ++p)
 	{
-		if ((setting = cJSON_GetObjectItem(level1, "alwaysalias"))) {
-			config.alwaysalias = (setting->type == cJSON_True);
-		}
-		if ((setting = cJSON_GetObjectItem(level1, "tracerlaser"))) {
-			weaponSystems[UNIT_LASER][SHOT_LASERTRACER] = (setting->type == cJSON_True);
-		}
-		if ((setting = cJSON_GetObjectItem(level1, "max_shots")) && setting->type == cJSON_Number) {
-			config.max_shots = setting->valueint;
-		}
-		if ((setting = cJSON_GetObjectItem(level1, "min_walls")) && setting->type == cJSON_Number) {
-			config.min_walls = setting->valueint;
-		}
-		if ((setting = cJSON_GetObjectItem(level1, "max_walls")) && setting->type == cJSON_Number) {
-			config.max_walls = setting->valueint;
-			if(config.min_walls > config.max_walls) { config.min_walls = config.max_walls; }
-		}
-		if ((setting = cJSON_GetObjectItem(level1, "max_explosions")) && setting->type == cJSON_Number) {
-			config.max_explosions = setting->valueint;
-		}
-
-		//player configuration
-		for(i = PLAYER_LEFT; i < MAX_PLAYERS; i++) {
-			sprintf(buffer, "player%i", i+1);
-			level2 = cJSON_GetObjectItem(level1, buffer);
-			if (level2 != NULL)
+		section = config_find_section(config_, "destruct keyboard", player_names[p]);
+		if (section == NULL)
+			if ((section = config_add_section(config_, "destruct keyboard", player_names[p])) == NULL)
+				exit(-1);
+		
+		ConfigOption *option;
+		
+		for (int k = 0; k < MAX_KEY; ++k)
+		{
+			if ((option = config_get_or_set_option(section, key_names[k], NULL)) == NULL)
+				exit(-1);
+			
+			foreach_option_i_value(i, value, option)
 			{
-				if ((setting = cJSON_GetObjectItem(level2, "jumper_fires_straight"))) {
-					config.jumper_straight[i] = (setting->type == cJSON_True);
-				}
-				if ((setting = cJSON_GetObjectItem(level2, "ai"))) {
-					config.ai[i] = (setting->type == cJSON_True);
-				}
-				//key configuration
-				level3 = cJSON_GetObjectItem(level2, "keys");
-				if (level3 != NULL)
+				SDL_Scancode key = SDL_GetScancodeFromName(value);
+				if (key != SDL_SCANCODE_UNKNOWN && i < COUNTOF(defaultKeyConfig[p][k]))
 				{
-					for (j = 0; j < COUNTOF(key_names); j++) {
-						for (k = 0; k < MAX_KEY_OPTIONS; k++) {
-							sprintf(buffer, "%s%i", key_names[j], k+1);
-							if ((setting = cJSON_GetObjectItem(level3, buffer)) && setting->type == cJSON_Number) {
-								defaultKeyConfig[i][j][k] = setting->valueint;
-							}
-							else { //assume that if we are reading keys the defaults are null and void
-								defaultKeyConfig[i][j][k] = SDLK_UNKNOWN;
-							}
-						}
-					}
+					defaultKeyConfig[p][k][i] = key;
+				}
+				else  // invalid or excess
+				{
+					foreach_remove_option_value();
+					continue;
 				}
 			}
-		}
-	}
-
-	//Now let's hit the custom mode...
-	level1 = cJSON_GetObjectItem(root, "custom");
-
-	if (level1 != NULL)
-	{
-		//general custom
-		if ((setting = cJSON_GetObjectItem(level1, "enable"))) {
-			config.allow_custom = (setting->type == cJSON_True);
-		}
-
-		//player configuration
-		for(i = PLAYER_LEFT; i < MAX_PLAYERS; i++) {
-			sprintf(buffer, "player%i", i+1);
-			level2 = cJSON_GetObjectItem(level1, buffer);
-			if (level2 != NULL)
+			
+			if (config_get_value_count(option) > 0)
 			{
-				if ((setting = cJSON_GetObjectItem(level2, "num_units"))) {
-					basetypes[8 + i][0] = setting->valueint;
-				}
-				for(j = 1; j < 11; j++) {
-					sprintf(buffer, "unit%i", j);
-					if ((setting = cJSON_GetObjectItem(level2, buffer)) && setting->type == cJSON_String) {
-						temp = string_to_unit_enum(setting->valuestring);
-						if(temp != UNIT_NONE) {
-							basetypes[8 + i][j] = temp;
-						}
-					}
-				}
+				// unset remaining defaults
+				for (unsigned int i = config_get_value_count(option); i < COUNTOF(defaultKeyConfig[p][k]); ++i)
+					defaultKeyConfig[p][k][i] = SDL_SCANCODE_UNKNOWN;
+			}
+			else
+			{
+				// set defaults
+				for (unsigned int i = 0; i < COUNTOF(defaultKeyConfig[p][k]); ++i)
+					if (defaultKeyConfig[p][k][i] != SDL_SCANCODE_UNKNOWN)
+						config_add_value(option, SDL_GetScancodeName(defaultKeyConfig[p][k][i]));
 			}
 		}
 	}
-
-	//wrap up
-	cJSON_Delete(root);
+	
+	// custom destruct mode
+	
+	section = config_find_section(config_, "destruct custom", NULL);
+	if (section == NULL)
+		if ((section = config_add_section(config_, "destruct custom", NULL)) == NULL)
+			exit(-1);
+	
+	config.allow_custom = config_get_or_set_bool_option(section, "enable", false, NO_YES);
+	
+	char buffer[15 + 1];
+	
+	for (int p = 0; p < MAX_PLAYERS; ++p)
+	{
+		snprintf(buffer, sizeof(buffer), "%s num units", player_names[p]);
+		basetypes[8 + p][0] = config_get_or_set_int_option(section, buffer, basetypes[8 + p][0]);
+		
+		ConfigOption *option;
+		
+		snprintf(buffer, sizeof(buffer), "%s unit", player_names[p]);
+		if ((option = config_get_or_set_option(section, buffer, NULL)) == NULL)
+			exit(-1);
+		
+		foreach_option_i_value(i, value, option)
+		{
+			enum de_unit_t unit = get_unit_by_name(value);
+			if (unit != UNIT_NONE && 1 + i < COUNTOF(basetypes[8 + p]))
+			{
+				basetypes[8 + p][1 + i] = unit;
+			}
+			else  // invalid or excess
+			{
+				foreach_remove_option_value();
+				continue;
+			}
+		}
+		
+		if (config_get_value_count(option) > 0)
+		{
+			// set remaining units to tank
+			for (unsigned int i = config_get_value_count(option); 1 + i < COUNTOF(basetypes[8 + p]); ++i)
+			{
+				basetypes[8 + p][1 + i] = UNIT_TANK;
+				config_add_value(option, unit_names[UNIT_TANK]);
+			}
+		}
+		else
+		{
+			// set defaults
+			for (unsigned int i = 0; 1 + i < COUNTOF(basetypes[8 + p]); ++i)
+				config_add_value(option, unit_names[basetypes[8 + p][1 + i]]);
+		}
+	}
 }
+
+/*** Startup ***/
+
 void JE_destructGame( void )
 {
 	unsigned int i;
@@ -690,7 +540,7 @@ void JE_destructGame( void )
 	JE_clr256(VGAScreen);
 	JE_showVGA();
 
-	load_destruct_config();
+	load_destruct_config(&opentyrian_config);
 
 	//malloc things that have customizable sizes
 	shotRec  = malloc(sizeof(struct destruct_shot_s)  * config.max_shots);
@@ -707,11 +557,14 @@ void JE_destructGame( void )
 	destructTempScreen = game_screen;
 	world.VGAScreen = VGAScreen;
 
-	JE_loadCompShapes(&eShapes[0], '~');
+	JE_loadCompShapes(&destructSpriteSheet, '~');
+
 	fade_black(1);
 
 	JE_destructMain();
 
+	free_sprite2s(&destructSpriteSheet);
+
 	//and of course exit actions go here.
 	free(shotRec);
 	free(exploRec);
@@ -829,16 +682,16 @@ static enum de_mode_t JE_modeSelect( void )
 		} while(!newkey);
 
 		/* See what was pressed */
-		if (keysactive[SDLK_ESCAPE])
+		if (keysactive[SDL_SCANCODE_ESCAPE])
 		{
 			mode = MODE_NONE; /* User is quitting, return failure */
 			break;
 		}
-		if (keysactive[SDLK_RETURN])
+		if (keysactive[SDL_SCANCODE_RETURN])
 		{
 			break; /* User has selected, return choice */
 		}
-		if (keysactive[SDLK_UP])
+		if (keysactive[SDL_SCANCODE_UP])
 		{
 			if(mode == MODE_FIRST)
 			{
@@ -852,7 +705,7 @@ static enum de_mode_t JE_modeSelect( void )
 				mode--;
 			}
 		}
-		if (keysactive[SDLK_DOWN])
+		if (keysactive[SDL_SCANCODE_DOWN])
 		{
 			if(mode >= MODE_LAST-1)
 			{
@@ -1191,8 +1044,8 @@ static void JE_aliasDirt( SDL_Surface * screen )
 	Uint8 *s = screen->pixels;
 	s += 12 * screen->pitch;
 
-	for (y = 12; y < (unsigned)screen->h; y++) {
-		for (x = 0; x < screen->pitch; x++) {
+	for (y = 12; y < (unsigned int)screen->h; y++) {
+		for (x = 0; x < (unsigned int)screen->pitch; x++) {
 			if (*s == PIXEL_BLACK) {
 				*s = aliasDirtPixel(screen, x, y, s);
 			}
@@ -1474,7 +1327,7 @@ static void JE_pauseScreen( void )
 
 /* DE_ResetX
  *
- * The reset functions clear the state of whatefer they are assigned to.
+ * The reset functions clear the state of whatever they are assigned to.
  */
 static void DE_ResetUnits( void )
 {
@@ -1619,39 +1472,39 @@ static enum de_state_t DE_RunTick( void )
 	DE_RunTickPlaySounds();
 
 	/* The rest of this cruft needs to be put in appropriate sections */
-	if (keysactive[SDLK_F10])
+	if (keysactive[SDL_SCANCODE_F10])
 	{
 		destruct_player[PLAYER_LEFT].is_cpu = !destruct_player[PLAYER_LEFT].is_cpu;
-		keysactive[SDLK_F10] = false;
+		keysactive[SDL_SCANCODE_F10] = false;
 	}
-	if (keysactive[SDLK_F11])
+	if (keysactive[SDL_SCANCODE_F11])
 	{
 		destruct_player[PLAYER_RIGHT].is_cpu = !destruct_player[PLAYER_RIGHT].is_cpu;
-		keysactive[SDLK_F11] = false;
+		keysactive[SDL_SCANCODE_F11] = false;
 	}
-	if (keysactive[SDLK_p])
+	if (keysactive[SDL_SCANCODE_P])
 	{
 		JE_pauseScreen();
-		keysactive[lastkey_sym] = false;
+		keysactive[lastkey_scan] = false;
 	}
 
-	if (keysactive[SDLK_F1])
+	if (keysactive[SDL_SCANCODE_F1])
 	{
 		JE_helpScreen();
-		keysactive[lastkey_sym] = false;
+		keysactive[lastkey_scan] = false;
 	}
 
 	wait_delay();
 
-	if (keysactive[SDLK_ESCAPE])
+	if (keysactive[SDL_SCANCODE_ESCAPE])
 	{
-		keysactive[SDLK_ESCAPE] = false;
+		keysactive[SDL_SCANCODE_ESCAPE] = false;
 		return(STATE_INIT); /* STATE_INIT drops us to the mode select */
 	}
 
-	if (keysactive[SDLK_BACKSPACE])
+	if (keysactive[SDL_SCANCODE_BACKSPACE])
 	{
-		keysactive[SDLK_BACKSPACE] = false;
+		keysactive[SDL_SCANCODE_BACKSPACE] = false;
 		return(STATE_RELOAD); /* STATE_RELOAD creates a new map */
 	}
 
@@ -1661,7 +1514,7 @@ static enum de_state_t DE_RunTick( void )
 /* DE_RunTickX
  *
  * Handles something that we do once per tick, such as
- * track ammo and move asplosions.
+ * track ammo and move explosions.
  */
 static void DE_RunTickCycleDeadUnits( void )
 {
@@ -1716,8 +1569,8 @@ static void DE_RunTickGravity( void )
 					DE_GravityFlyUnit(unit);
 					break;
 				}
-				/* else fall through and treat as a normal unit */
-
+				/* else treat as a normal unit */
+				/* fall through */
 			default:
 				DE_GravityLowerUnit(unit);
 			}
@@ -1735,7 +1588,7 @@ static void DE_GravityDrawUnit( enum de_player_t team, struct destruct_unit_s *
 	anim_index = GraphicBase[team][unit->unitType] + unit->ani_frame;
 	if (unit->unitType == UNIT_HELI)
 	{
-		/* Adjust animation index if we are travelling right or left. */
+		/* Adjust animation index if we are traveling right or left. */
 		if (unit->lastMove < -2)
 			anim_index += 5;
 		else if (unit->lastMove > 2)
@@ -1746,7 +1599,7 @@ static void DE_GravityDrawUnit( enum de_player_t team, struct destruct_unit_s *
 		anim_index += floorf(unit->angle * 9.99f / M_PI);
 	}
 
-	blit_sprite2(VGAScreen, unit->unitX, roundf(unit->unitY) - 13, eShapes[0], anim_index);
+	blit_sprite2(VGAScreen, unit->unitX, roundf(unit->unitY) - 13, destructSpriteSheet, anim_index);
 }
 static void DE_GravityLowerUnit( struct destruct_unit_s * unit )
 {
@@ -1838,7 +1691,7 @@ static void DE_RunTickDrawWalls( void )
 	{
 		if (world.mapWalls[i].wallExist)
 		{
-			blit_sprite2(VGAScreen, world.mapWalls[i].wallX, world.mapWalls[i].wallY, eShapes[0], 42);
+			blit_sprite2(VGAScreen, world.mapWalls[i].wallX, world.mapWalls[i].wallY, destructSpriteSheet, 42);
 		}
 	}
 }
@@ -2240,9 +2093,9 @@ static void DE_RunTickAI( void )
 			ptrUnit = ptrTarget->unit;
 			for (j = 0; j < config.max_installations; j++, ptrUnit++)
 			{
-				if (abs(ptrUnit->unitX - ptrCurUnit->unitX) < 8)
+				if (abs((int)ptrUnit->unitX - (int)ptrCurUnit->unitX) < 8)
 				{
-					/* I get it.  This makes helicoptors hover over
+					/* I get it.  This makes helicopters hover over
 					 * their enemies. */
 					if (ptrUnit->unitType == UNIT_SATELLITE)
 					{
@@ -2377,7 +2230,7 @@ static void DE_RunTickDrawHUD( void )
 		JE_rectangle(VGAScreen, startX + 17, 2, startX + 143, 9, 242);
 		JE_rectangle(VGAScreen, startX + 16, 1, startX + 144, 10, 240);
 
-		blit_sprite2(VGAScreen, startX +  4, 0, eShapes[0], 191 + curUnit->shotType);
+		blit_sprite2(VGAScreen, startX +  4, 0, destructSpriteSheet, 191 + curUnit->shotType);
 
 		JE_outText   (VGAScreen, startX + 20, 3, weaponNames[curUnit->shotType], 15, 2);
 		sprintf      (tempstr, "dmg~%d~", curUnit->health);
@@ -2389,7 +2242,7 @@ static void DE_RunTickDrawHUD( void )
 static void DE_RunTickGetInput( void )
 {
 	unsigned int player_index, key_index, slot_index;
-	SDLKey key;
+	SDL_Scancode key;
 
 	/* destruct_player.keys holds our key config.  Players will eventually be
 	 * allowed to can change their key mappings.  destruct_player.moves and
@@ -2405,7 +2258,7 @@ static void DE_RunTickGetInput( void )
 			for(slot_index = 0; slot_index < MAX_KEY_OPTIONS; slot_index++)
 			{
 				key = destruct_player[player_index].keys.Config[key_index][slot_index];
-				if(key == SDLK_UNKNOWN) { break; }
+				if(key == SDL_SCANCODE_UNKNOWN) { break; }
 				if(keysactive[key] == true)
 				{
 					/* The right key was clearly pressed */
