@@ -1,4 +1,4 @@
-/* 
+/*
  * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
@@ -17,12 +17,18 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 #include "file.h"
+
 #include "opentyr.h"
+#include "varz.h"
 
 #include "SDL.h"
-#include <errno.h>
 
-const char *custom_data_dir = ".";
+#include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+const char *custom_data_dir = NULL;
 
 // finds the Tyrian data directory
 const char *data_dir( void )
@@ -30,33 +36,34 @@ const char *data_dir( void )
 	const char *dirs[] =
 	{
 		custom_data_dir,
+		TYRIAN_DIR,
 		"data",
-#ifdef TARGET_MACOSX
-		tyrian_game_folder(),
-#endif
-		"/usr/share/opentyrian/data"
+		".",
 	};
-	
+
 	static const char *dir = NULL;
-	
+
 	if (dir != NULL)
 		return dir;
-	
+
 	for (uint i = 0; i < COUNTOF(dirs); ++i)
 	{
+		if (dirs[i] == NULL)
+			continue;
+
 		FILE *f = dir_fopen(dirs[i], "tyrian1.lvl", "rb");
 		if (f)
 		{
 			fclose(f);
-			
+
 			dir = dirs[i];
 			break;
 		}
 	}
-	
+
 	if (dir == NULL) // data not found
 		dir = "";
-	
+
 	return dir;
 }
 
@@ -65,11 +72,11 @@ FILE *dir_fopen( const char *dir, const char *file, const char *mode )
 {
 	char *path = malloc(strlen(dir) + 1 + strlen(file) + 1);
 	sprintf(path, "%s/%s", dir, file);
-	
+
 	FILE *f = fopen(path, mode);
-	
+
 	free(path);
-	
+
 	return f;
 }
 
@@ -77,10 +84,10 @@ FILE *dir_fopen( const char *dir, const char *file, const char *mode )
 FILE *dir_fopen_warn(  const char *dir, const char *file, const char *mode )
 {
 	FILE *f = dir_fopen(dir, file, mode);
-	
+
 	if (f == NULL)
 		fprintf(stderr, "warning: failed to open '%s': %s\n", file, strerror(errno));
-	
+
 	return f;
 }
 
@@ -88,15 +95,15 @@ FILE *dir_fopen_warn(  const char *dir, const char *file, const char *mode )
 FILE *dir_fopen_die( const char *dir, const char *file, const char *mode )
 {
 	FILE *f = dir_fopen(dir, file, mode);
-	
+
 	if (f == NULL)
 	{
 		fprintf(stderr, "error: failed to open '%s': %s\n", file, strerror(errno));
 		fprintf(stderr, "error: One or more of the required Tyrian " TYRIAN_VERSION " data files could not be found.\n"
 		                "       Please read the README file.\n");
-		exit(1);
+		JE_tyrianHalt(1);
 	}
-	
+
 	return f;
 }
 
@@ -113,74 +120,33 @@ bool dir_file_exists( const char *dir, const char *file )
 long ftell_eof( FILE *f )
 {
 	long pos = ftell(f);
-	
+
 	fseek(f, 0, SEEK_END);
 	long size = ftell(f);
-	
+
 	fseek(f, pos, SEEK_SET);
-	
+
 	return size;
 }
 
-#if SDL_BYTEORDER == SDL_BIG_ENDIAN
-// endian-swapping fread
-size_t efread( void *buffer, size_t size, size_t num, FILE *stream )
+void fread_die(void *buffer, size_t size, size_t count, FILE *stream)
 {
-	size_t f = fread(buffer, size, num, stream);
-	
-	switch (size)
+	size_t result = fread(buffer, size, count, stream);
+	if (result != count)
 	{
-		case 2:
-			for (size_t i = 0; i < num; i++)
-				((Uint16 *)buffer)[i] = SDL_Swap16(((Uint16 *)buffer)[i]);
-			break;
-		case 4:
-			for (size_t i = 0; i < num; i++)
-				((Uint32 *)buffer)[i] = SDL_Swap32(((Uint32 *)buffer)[i]);
-			break;
-		case 8:
-			for (size_t i = 0; i < num; i++)
-				((Uint64 *)buffer)[i] = SDL_Swap64(((Uint64 *)buffer)[i]);
-			break;
-		default:
-			break;
+		fprintf(stderr, "error: An unexpected problem occurred while reading from a file.\n");
+		SDL_Quit();
+		exit(EXIT_FAILURE);
 	}
-	
-	return f;
 }
 
-// endian-swapping fwrite
-size_t efwrite( void *buffer, size_t size, size_t num, FILE *stream )
+void fwrite_die(const void *buffer, size_t size, size_t count, FILE *stream)
 {
-	void *swap_buffer;
-	
-	switch (size)
+	size_t result = fwrite(buffer, size, count, stream);
+	if (result != count)
 	{
-		case 2:
-			swap_buffer = malloc(size * num);
-			for (size_t i = 0; i < num; i++)
-				((Uint16 *)swap_buffer)[i] = SDL_SwapLE16(((Uint16 *)buffer)[i]);
-			break;
-		case 4:
-			swap_buffer = malloc(size * num);
-			for (size_t i = 0; i < num; i++)
-				((Uint32 *)swap_buffer)[i] = SDL_SwapLE32(((Uint32 *)buffer)[i]);
-			break;
-		case 8:
-			swap_buffer = malloc(size * num);
-			for (size_t i = 0; i < num; i++)
-				((Uint64 *)swap_buffer)[i] = SDL_SwapLE64(((Uint64 *)buffer)[i]);
-			break;
-		default:
-			swap_buffer = buffer;
-			break;
+		fprintf(stderr, "error: An unexpected problem occurred while writing to a file.\n");
+		SDL_Quit();
+		exit(EXIT_FAILURE);
 	}
-	
-	size_t f = fwrite(swap_buffer, size, num, stream);
-	
-	if (swap_buffer != buffer)
-		free(swap_buffer);
-	
-	return f;
 }
-#endif
