@@ -21,6 +21,10 @@
 
 #include "opentyr.h"
 
+#include "SDL.h"
+
+#include <stdio.h>
+
 #define MENU_MAX 14
 
 #define DESTRUCT_MODES 5
@@ -30,20 +34,6 @@ extern const JE_byte menuHelp[MENU_MAX][11];   /* [1..14, 1..11] */
 extern JE_byte verticalHeight;
 extern JE_byte helpBoxColor, helpBoxBrightness, helpBoxShadeType;
 
-#ifdef TYRIAN2000
-#define HELPTEXT_MISCTEXT_COUNT 72
-#define HELPTEXT_MISCTEXTB_COUNT 8
-#define HELPTEXT_MISCTEXTB_SIZE 12
-#define HELPTEXT_MENUTEXT_SIZE 29
-#define HELPTEXT_MAINMENUHELP_COUNT 37
-#define HELPTEXT_NETWORKTEXT_COUNT 5
-#define HELPTEXT_NETWORKTEXT_SIZE 33
-#define HELPTEXT_SUPERSHIPS_COUNT 13
-#define HELPTEXT_SPECIALNAME_COUNT 11
-#define HELPTEXT_SHIPINFO_COUNT 20
-#define HELPTEXT_MENUINT3_COUNT 9
-#define HELPTEXT_MENUINT12_COUNT 7
-#else
 #define HELPTEXT_MISCTEXT_COUNT 68
 #define HELPTEXT_MISCTEXTB_COUNT 5
 #define HELPTEXT_MISCTEXTB_SIZE 11
@@ -54,7 +44,6 @@ extern JE_byte helpBoxColor, helpBoxBrightness, helpBoxShadeType;
 #define HELPTEXT_SUPERSHIPS_COUNT 11
 #define HELPTEXT_SPECIALNAME_COUNT 9
 #define HELPTEXT_SHIPINFO_COUNT 13
-#endif
 
 extern char helpTxt[39][231];
 extern char pName[21][16];
@@ -80,7 +69,7 @@ extern char destructModeName[DESTRUCT_MODES][13];
 extern char shipInfo[HELPTEXT_SHIPINFO_COUNT][2][256];
 extern char menuInt[MENU_MAX+1][11][18];
 
-void read_encrypted_pascal_string( char *s, int size, FILE *f );
+void read_encrypted_pascal_string( char *s, size_t size, FILE *f );
 void skip_pascal_string( FILE *f );
 
 void JE_helpBox( SDL_Surface *screen, int x, int y, const char *message, unsigned int boxwidth );
