@@ -16,10 +16,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
-#include "cJSON.h"
-#include "config.h"
-#include "file.h"
 #include "joystick.h"
+
+#include "config.h"
+#include "config_file.h"
+#include "file.h"
 #include "keyboard.h"
 #include "nortsong.h"
 #include "opentyr.h"
@@ -28,6 +29,8 @@
 #include "video.h"
 
 #include <assert.h>
+#include <ctype.h>
+#include <string.h>
 
 int joystick_axis_threshold( int j, int value );
 int check_assigned( SDL_Joystick *joystick_handle, const Joystick_assignment assignment[2] );
@@ -42,7 +45,6 @@ bool ignore_joystick = false;
 int joysticks = 0;
 Joystick *joystick = NULL;
 
-static const char joystick_cfg_version = 1;
 static const int joystick_analog_max = 32767;
 
 // eliminates axis movement below the threshold
@@ -187,7 +189,7 @@ void poll_joystick( int j )
 	{
 		bool old = joystick[j].action[d];
 		
-		joystick[j].action[d] = check_assigned(joystick[j].handle, joystick[j].assignment[d + COUNTOF(joystick[j].direction)]);
+		joystick[j].action[d] = check_assigned(joystick[j].handle, joystick[j].assignment[d + COUNTOF(joystick[j].direction)]) > (joystick_analog_max / 2);
 		joydown |= joystick[j].action[d];
 		
 		joystick[j].action_pressed[d] = joystick[j].action[d] && (!old || repeat);
@@ -212,15 +214,13 @@ void poll_joysticks( void )
 }
 
 // sends SDL KEYDOWN and KEYUP events for a key
-void push_key( SDLKey key )
+void push_key( SDL_Scancode key )
 {
 	SDL_Event e;
 	
 	memset(&e.key.keysym, 0, sizeof(e.key.keysym));
 	
-	e.key.keysym.sym = key;
-	e.key.keysym.unicode = key;
-	
+	e.key.keysym.scancode = key;
 	e.key.state = SDL_RELEASED;
 	
 	e.type = SDL_KEYDOWN;
@@ -233,8 +233,8 @@ void push_key( SDLKey key )
 // helps us be lazy by pretending joysticks are a keyboard (useful for menus)
 void push_joysticks_as_keyboard( void )
 {
-	const SDLKey confirm = SDLK_RETURN, cancel = SDLK_ESCAPE;
-	const SDLKey direction[4] = { SDLK_UP, SDLK_RIGHT, SDLK_DOWN, SDLK_LEFT };
+	const SDL_Scancode confirm = SDL_SCANCODE_RETURN, cancel = SDL_SCANCODE_ESCAPE;
+	const SDL_Scancode direction[4] = { SDL_SCANCODE_UP, SDL_SCANCODE_RIGHT, SDL_SCANCODE_DOWN, SDL_SCANCODE_LEFT };
 	
 	poll_joysticks();
 	
@@ -281,13 +281,13 @@ void init_joysticks( void )
 		joystick[j].handle = SDL_JoystickOpen(j);
 		if (joystick[j].handle != NULL)
 		{
-			printf("joystick detected: %s ", SDL_JoystickName(j));
+			printf("joystick detected: %s ", SDL_JoystickName(joystick[j].handle));
 			printf("(%d axes, %d buttons, %d hats)\n", 
 			       SDL_JoystickNumAxes(joystick[j].handle),
 			       SDL_JoystickNumButtons(joystick[j].handle),
 			       SDL_JoystickNumHats(joystick[j].handle));
 			
-			if (!load_joystick_assignments(j))
+			if (!load_joystick_assignments(&opentyrian_config, j))
 				reset_joystick_assignments(j);
 		}
 	}
@@ -306,7 +306,7 @@ void deinit_joysticks( void )
 	{
 		if (joystick[j].handle != NULL)
 		{
-			save_joystick_assignments(j);
+			save_joystick_assignments(&opentyrian_config, j);
 			SDL_JoystickClose(joystick[j].handle);
 		}
 	}
@@ -359,94 +359,87 @@ void reset_joystick_assignments( int j )
 	joystick[j].threshold = 5;
 }
 
-bool load_joystick_assignments( int j )
+static const char* const assignment_names[] =
 {
-	cJSON *root = load_json("joystick.conf");
-	if (root == NULL)
+	"up",
+	"right",
+	"down",
+	"left",
+	"fire",
+	"change fire",
+	"left sidekick",
+	"right sidekick",
+	"menu",
+	"pause",
+};
+
+bool load_joystick_assignments( Config *config, int j )
+{
+	ConfigSection *section = config_find_section(config, "joystick", SDL_JoystickName(joystick[j].handle));
+	if (section == NULL)
 		return false;
 	
-	cJSON *config = cJSON_GetObjectItem(root, SDL_JoystickName(j));
-	if (config == NULL)
+	if (!config_get_bool_option(section, "analog", &joystick[j].analog))
+		joystick[j].analog = false;
+	
+	joystick[j].sensitivity = config_get_or_set_int_option(section, "sensitivity", 5);
+
+	joystick[j].threshold = config_get_or_set_int_option(section, "threshold", 5);
+	
+	for (size_t a = 0; a < COUNTOF(assignment_names); ++a)
 	{
-		cJSON_Delete(root);
-		return false;
-	}
-	
-	cJSON *setting;
-	
-	if ((setting = cJSON_GetObjectItem(config, "analog")))
-		joystick[j].analog = (setting->type == cJSON_True);
-	
-	if ((setting = cJSON_GetObjectItem(config, "sensitivity")))
-		joystick[j].sensitivity = setting->valueint;
-	
-	if ((setting = cJSON_GetObjectItem(config, "threshold")))
-		joystick[j].threshold = setting->valueint;
-	
-	if ((setting = cJSON_GetObjectItem(config, "assignments")))
-	{
-		for (uint i = 0; i < COUNTOF(joystick->assignment); ++i)
+		for (unsigned int i = 0; i < COUNTOF(joystick[j].assignment[a]); ++i)
+			joystick[j].assignment[a][i].type = NONE;
+		
+		ConfigOption *option = config_get_option(section, assignment_names[a]);
+		if (option == NULL)
+			continue;
+		
+		foreach_option_i_value(i, value, option)
 		{
-			cJSON *assignments = cJSON_GetArrayItem(setting, i);
-			if (assignments == NULL)
+			if (i >= COUNTOF(joystick[j].assignment[a]))
 				break;
 			
-			for (uint k = 0; k < COUNTOF(*joystick->assignment); ++k)
-			{
-				cJSON *assignment = cJSON_GetArrayItem(assignments, k);
-				if (assignment)
-					code_to_assignment(&joystick[j].assignment[i][k], assignment->valuestring);
-			}
+			code_to_assignment(&joystick[j].assignment[a][i], value);
 		}
 	}
 	
-	cJSON_Delete(root);
-	
 	return true;
 }
 
-bool save_joystick_assignments( int j )
+bool save_joystick_assignments( Config *config, int j )
 {
-	cJSON *root = load_json("joystick.conf");
-	if (root == NULL)
-		root = cJSON_CreateObject();
+	ConfigSection *section = config_find_or_add_section(config, "joystick", SDL_JoystickName(joystick[j].handle));
+	if (section == NULL)
+		exit(EXIT_FAILURE);  // out of memory
 	
-	cJSON *config = cJSON_CreateOrGetObjectItem(root, SDL_JoystickName(j));
-	cJSON_ForceType(config, cJSON_Object);
+	config_set_bool_option(section, "analog", joystick[j].analog, NO_YES);
 	
-	cJSON *setting;
+	config_set_int_option(section, "sensitivity", joystick[j].sensitivity);
 	
-	setting = cJSON_CreateOrGetObjectItem(config, "analog");
-	cJSON_SetBoolean(setting, joystick[j].analog);
+	config_set_int_option(section, "threshold", joystick[j].threshold);
 	
-	setting = cJSON_CreateOrGetObjectItem(config, "sensitivity");
-	cJSON_SetNumber(setting, joystick[j].sensitivity);
-	
-	setting = cJSON_CreateOrGetObjectItem(config, "threshold");
-	cJSON_SetNumber(setting, joystick[j].threshold);
-	
-	setting = cJSON_CreateOrGetObjectItem(config, "assignments");
-	cJSON_ForceType(setting, cJSON_Array);
-	cJSON_ClearArray(setting);
-	
-	for (uint i = 0; i < COUNTOF(joystick->assignment); ++i)
+	for (size_t a = 0; a < COUNTOF(assignment_names); ++a)
 	{
-		cJSON *assignments;
-		cJSON_AddItemToArray(setting, assignments = cJSON_CreateArray());
+		ConfigOption *option = config_set_option(section, assignment_names[a], NULL);
+		if (option == NULL)
+			exit(EXIT_FAILURE);  // out of memory
 		
-		for (uint k = 0; k < COUNTOF(*joystick->assignment); ++k)
+		option = config_set_value(option, NULL);
+		if (option == NULL)
+			exit(EXIT_FAILURE);  // out of memory
+
+		for (size_t i = 0; i < COUNTOF(joystick[j].assignment[a]); ++i)
 		{
-			if (joystick[j].assignment[i][k].type == NONE)
+			if (joystick[j].assignment[a][i].type == NONE)
 				continue;
 			
-			cJSON_AddItemToArray(assignments, cJSON_CreateString(assignment_to_code(&joystick[j].assignment[i][k])));
+			option = config_add_value(option, assignment_to_code(&joystick[j].assignment[a][i]));
+			if (option == NULL)
+				exit(EXIT_FAILURE);  // out of memory
 		}
 	}
 	
-	save_json(root, "joystick.conf");
-	
-	cJSON_Delete(root);
-	
 	return true;
 }
 
