@@ -1,4 +1,4 @@
-/* 
+/*
  * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
@@ -16,8 +16,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
-#include "joystick.h"
 #include "keyboard.h"
+
+#include "joystick.h"
+#include "mouse.h"
 #include "network.h"
 #include "opentyr.h"
 #include "video.h"
@@ -25,26 +27,30 @@
 
 #include "SDL.h"
 
+#include <stdio.h>
 
 JE_boolean ESCPressed;
 
 JE_boolean newkey, newmouse, keydown, mousedown;
-SDLKey lastkey_sym;
-SDLMod lastkey_mod;
-unsigned char lastkey_char;
+SDL_Scancode lastkey_scan;
+SDL_Keymod lastkey_mod;
 Uint8 lastmouse_but;
-Uint16 lastmouse_x, lastmouse_y;
+Sint32 lastmouse_x, lastmouse_y;
 JE_boolean mouse_pressed[3] = {false, false, false};
-Uint16 mouse_x, mouse_y;
+Sint32 mouse_x, mouse_y;
 
-Uint8 keysactive[SDLK_LAST];
+bool windowHasFocus;
 
-#ifdef NDEBUG
-bool input_grab_enabled = true;
-#else
-bool input_grab_enabled = false;
-#endif
+Uint8 keysactive[SDL_NUM_SCANCODES];
 
+bool new_text;
+char last_text[SDL_TEXTINPUTEVENT_TEXT_SIZE];
+
+static bool mouseRelativeEnabled;
+
+// Relative mouse position in window coordinates.
+static Sint32 mouseWindowXRelative;
+static Sint32 mouseWindowYRelative;
 
 void flush_events_buffer( void )
 {
@@ -61,7 +67,7 @@ void wait_input( JE_boolean keyboard, JE_boolean mouse, JE_boolean joystick )
 		SDL_Delay(SDL_POLL_INTERVAL);
 		push_joysticks_as_keyboard();
 		service_SDL_events(false);
-		
+
 #ifdef WITH_NETWORK
 		if (isNetworkGame)
 			network_check();
@@ -77,7 +83,7 @@ void wait_noinput( JE_boolean keyboard, JE_boolean mouse, JE_boolean joystick )
 		SDL_Delay(SDL_POLL_INTERVAL);
 		poll_joysticks();
 		service_SDL_events(false);
-		
+
 #ifdef WITH_NETWORK
 		if (isNetworkGame)
 			network_check();
@@ -87,26 +93,22 @@ void wait_noinput( JE_boolean keyboard, JE_boolean mouse, JE_boolean joystick )
 
 void init_keyboard( void )
 {
-	SDL_EnableKeyRepeat(500, 60);
+	//SDL_EnableKeyRepeat(500, 60); TODO Find if SDL2 has an equivalent.
 
 	newkey = newmouse = false;
 	keydown = mousedown = false;
 
-	SDL_EnableUNICODE(1);
+	SDL_ShowCursor(SDL_FALSE);
 }
 
-void input_grab( bool enable )
+void mouseSetRelative(bool enable)
 {
-#if defined(TARGET_GP2X) || defined(TARGET_DINGUX)
-	enable = true;
-#endif
-	
-	input_grab_enabled = enable || fullscreen_enabled;
-	
-	SDL_ShowCursor(input_grab_enabled ? SDL_DISABLE : SDL_ENABLE);
-#ifdef NDEBUG
-	SDL_WM_GrabInput(input_grab_enabled ? SDL_GRAB_ON : SDL_GRAB_OFF);
-#endif
+	SDL_SetRelativeMouseMode(enable && windowHasFocus);
+
+	mouseRelativeEnabled = enable;
+
+	mouseWindowXRelative = 0;
+	mouseWindowYRelative = 0;
 }
 
 JE_word JE_mousePosition( JE_word *mouseX, JE_word *mouseY )
@@ -117,110 +119,108 @@ JE_word JE_mousePosition( JE_word *mouseX, JE_word *mouseY )
 	return mousedown ? lastmouse_but : 0;
 }
 
-void set_mouse_position( int x, int y )
+void mouseGetRelativePosition(Sint32 *const out_x, Sint32 *const out_y)
 {
-	if (input_grab_enabled)
-	{
-		SDL_WarpMouse(x * scalers[scaler].width / vga_width, y * scalers[scaler].height / vga_height);
-		mouse_x = x;
-		mouse_y = y;
-	}
+	service_SDL_events(false);
+
+	scaleWindowDistanceToScreen(&mouseWindowXRelative, &mouseWindowYRelative);
+	*out_x = mouseWindowXRelative;
+	*out_y = mouseWindowYRelative;
+
+	mouseWindowXRelative = 0;
+	mouseWindowYRelative = 0;
 }
 
 void service_SDL_events( JE_boolean clear_new )
 {
 	SDL_Event ev;
-	
+
 	if (clear_new)
-		newkey = newmouse = false;
-	
+	{
+		newkey = false;
+		newmouse = false;
+		new_text = false;
+	}
+
 	while (SDL_PollEvent(&ev))
 	{
 		switch (ev.type)
 		{
-			case SDL_ACTIVEEVENT:
-				if (ev.active.state == SDL_APPINPUTFOCUS && !ev.active.gain)
-					input_grab(false);
-				break;
-			
-			case SDL_MOUSEMOTION:
-				mouse_x = ev.motion.x * vga_width / scalers[scaler].width;
-				mouse_y = ev.motion.y * vga_height / scalers[scaler].height;
-				break;
-			case SDL_KEYDOWN:
-				if (ev.key.keysym.mod & KMOD_CTRL)
+			case SDL_WINDOWEVENT:
+				switch (ev.window.event)
 				{
-					/* <ctrl><bksp> emergency kill */
-					if (ev.key.keysym.sym == SDLK_BACKSPACE)
-					{
-						puts("\n\n\nCtrl+Backspace pressed. Doing emergency quit.\n");
-						SDL_Quit();
-						exit(1);
-					}
-					
-					/* <ctrl><f10> toggle input grab */
-					if (ev.key.keysym.sym == SDLK_F10)
-					{
-						input_grab(!input_grab_enabled);
-						break;
-					}
-				}
-				
-				if (ev.key.keysym.mod & KMOD_ALT)
-				{
-					/* <alt><enter> toggle fullscreen */
-					if (ev.key.keysym.sym == SDLK_RETURN)
-					{
-						if (!init_scaler(scaler, !fullscreen_enabled) && // try new fullscreen state
-						    !init_any_scaler(!fullscreen_enabled) &&     // try any scaler in new fullscreen state
-						    !init_scaler(scaler, fullscreen_enabled))    // revert on fail
-						{
-							exit(EXIT_FAILURE);
-						}
-						break;
-					}
-					
-					/* <alt><tab> disable input grab and fullscreen */
-					if (ev.key.keysym.sym == SDLK_TAB)
-					{
-						if (!init_scaler(scaler, false) &&             // try windowed
-						    !init_any_scaler(false) &&                 // try any scaler windowed
-						    !init_scaler(scaler, fullscreen_enabled))  // revert on fail
-						{
-							exit(EXIT_FAILURE);
-						}
-						
-						input_grab(false);
-						break;
-					}
-				}
+				case SDL_WINDOWEVENT_FOCUS_LOST:
+					windowHasFocus = false;
 
-				keysactive[ev.key.keysym.sym] = 1;
-				
-				newkey = true;
-				lastkey_sym = ev.key.keysym.sym;
-				lastkey_mod = ev.key.keysym.mod;
-				lastkey_char = ev.key.keysym.unicode;
-				keydown = true;
-				return;
-			case SDL_KEYUP:
-				keysactive[ev.key.keysym.sym] = 0;
-				keydown = false;
-				return;
-			case SDL_MOUSEBUTTONDOWN:
-				if (!input_grab_enabled)
-				{
-					input_grab(true);
+					mouseSetRelative(mouseRelativeEnabled);
+					break;
+
+				case SDL_WINDOWEVENT_FOCUS_GAINED:
+					windowHasFocus = true;
+
+					mouseSetRelative(mouseRelativeEnabled);
+					break;
+
+				case SDL_WINDOWEVENT_RESIZED:
+					video_on_win_resize();
 					break;
 				}
-				// intentional fall-though
+				break;
+
+			case SDL_KEYDOWN:
+				/* <alt><enter> toggle fullscreen */
+				if (ev.key.keysym.mod & KMOD_ALT && ev.key.keysym.scancode == SDL_SCANCODE_RETURN)
+				{
+					toggle_fullscreen();
+					break;
+				}
+
+				keysactive[ev.key.keysym.scancode] = 1;
+
+				newkey = true;
+				lastkey_scan = ev.key.keysym.scancode;
+				lastkey_mod = ev.key.keysym.mod;
+				keydown = true;
+
+				mouseInactive = true;
+				return;
+
+			case SDL_KEYUP:
+				keysactive[ev.key.keysym.scancode] = 0;
+				keydown = false;
+				return;
+
+			case SDL_MOUSEMOTION:
+				mouse_x = ev.motion.x;
+				mouse_y = ev.motion.y;
+				mapWindowPointToScreen(&mouse_x, &mouse_y);
+
+				if (mouseRelativeEnabled && windowHasFocus)
+				{
+					mouseWindowXRelative += ev.motion.xrel;
+					mouseWindowYRelative += ev.motion.yrel;
+				}
+
+				// Show system mouse pointer if outside screen.
+				SDL_ShowCursor(mouse_x < 0 || mouse_x >= vga_width ||
+				               mouse_y < 0 || mouse_y >= vga_height ? SDL_TRUE : SDL_FALSE);
+
+				if (ev.motion.xrel != 0 || ev.motion.yrel != 0)
+					mouseInactive = false;
+				break;
+
+			case SDL_MOUSEBUTTONDOWN:
+				mouseInactive = false;
+
+				// fall through
 			case SDL_MOUSEBUTTONUP:
+				mapWindowPointToScreen(&ev.button.x, &ev.button.y);
 				if (ev.type == SDL_MOUSEBUTTONDOWN)
 				{
 					newmouse = true;
 					lastmouse_but = ev.button.button;
-					lastmouse_x = ev.button.x * vga_width / scalers[scaler].width;
-					lastmouse_y = ev.button.y * vga_height / scalers[scaler].height;
+					lastmouse_x = ev.button.x;
+					lastmouse_y = ev.button.y;
 					mousedown = true;
 				}
 				else
@@ -237,6 +237,15 @@ void service_SDL_events( JE_boolean clear_new )
 						mouse_pressed[2] = mousedown; break;
 				}
 				break;
+
+			case SDL_TEXTINPUT:
+				SDL_strlcpy(last_text, ev.text.text, COUNTOF(last_text));
+				new_text = true;
+				break;
+
+			case SDL_TEXTEDITING:
+				break;
+
 			case SDL_QUIT:
 				/* TODO: Call the cleanup code here. */
 				exit(0);
