@@ -23,29 +23,32 @@
 
 #include "SDL.h"
 
+#include <stdbool.h>
 
 #define SDL_POLL_INTERVAL 5
 
 extern JE_boolean ESCPressed;
 extern JE_boolean newkey, newmouse, keydown, mousedown;
-extern SDLKey lastkey_sym;
-extern SDLMod lastkey_mod;
-extern unsigned char lastkey_char;
+extern SDL_Scancode lastkey_scan;
+extern SDL_Keymod lastkey_mod;
 extern Uint8 lastmouse_but;
-extern Uint16 lastmouse_x, lastmouse_y;
+extern Sint32 lastmouse_x, lastmouse_y;
 extern JE_boolean mouse_pressed[3];
-extern Uint16 mouse_x, mouse_y;
-extern Uint8 keysactive[SDLK_LAST];
+extern Sint32 mouse_x, mouse_y;
+extern Uint8 keysactive[SDL_NUM_SCANCODES];
 
-extern bool input_grab_enabled;
+extern bool windowHasFocus;
+
+extern bool new_text;
+extern char last_text[SDL_TEXTINPUTEVENT_TEXT_SIZE];
 
 void flush_events_buffer( void );
 void wait_input( JE_boolean keyboard, JE_boolean mouse, JE_boolean joystick );
 void wait_noinput( JE_boolean keyboard, JE_boolean mouse, JE_boolean joystick );
 void init_keyboard( void );
-void input_grab( bool enable );
+void mouseSetRelative(bool enable);
 JE_word JE_mousePosition( JE_word *mouseX, JE_word *mouseY );
-void set_mouse_position( int x, int y );
+void mouseGetRelativePosition(Sint32 *out_x, Sint32 *out_y);
 
 void service_SDL_events( JE_boolean clear_new );
 
