@@ -16,13 +16,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+#include "loudness.h"
+
 #include "file.h"
 #include "lds_play.h"
-#include "loudness.h"
 #include "nortsong.h"
 #include "opentyr.h"
 #include "params.h"
 
+#include <stdlib.h>
+
 float music_volume = 0, sample_volume = 0;
 
 bool music_stopped = true;
@@ -42,11 +45,10 @@ Uint32 channel_len[SFX_CHANNELS] = { 0 };
 Uint8 channel_vol[SFX_CHANNELS];
 
 int sound_init_state = false;
-int freq = 11025 * OUTPUT_QUALITY;
 
-static SDL_AudioCVT audio_cvt; // used for format conversion
+static SDL_AudioDeviceID audio_device = 0;
 
-void audio_cb( void *userdata, unsigned char *feedme, int howmuch );
+void audio_cb( void *user_data, unsigned char *sdl_buffer, int bytes_needed );
 
 void load_song( unsigned int song_num );
 
@@ -57,59 +59,58 @@ bool init_audio( void )
 	
 	SDL_AudioSpec ask, got;
 	
-	ask.freq = freq;
+	ask.freq = SAMPLE_RATE;
 	ask.format = (BYTES_PER_SAMPLE == 2) ? AUDIO_S16SYS : AUDIO_S8;
 	ask.channels = 1;
-	ask.samples = 2048;
+	ask.samples = 256 * OUTPUT_QUALITY; // ~23 ms
 	ask.callback = audio_cb;
 	
-	printf("\trequested %d Hz, %d channels, %d samples\n", ask.freq, ask.channels, ask.samples);
-	
-	if (SDL_OpenAudio(&ask, &got) == -1)
+	if (SDL_InitSubSystem(SDL_INIT_AUDIO))
 	{
 		fprintf(stderr, "error: failed to initialize SDL audio: %s\n", SDL_GetError());
 		audio_disabled = true;
 		return false;
 	}
 	
-	printf("\tobtained  %d Hz, %d channels, %d samples\n", got.freq, got.channels, got.samples);
-	
-	SDL_BuildAudioCVT(&audio_cvt, ask.format, ask.channels, ask.freq, got.format, got.channels, got.freq);
+	audio_device = SDL_OpenAudioDevice(/*device*/ NULL, /*iscapture*/ 0, &ask, &got, /*allowed_changes*/ 0);
+
+	if (audio_device == 0)
+	{
+		fprintf(stderr, "error: SDL failed to open audio device: %s\n", SDL_GetError());
+		audio_disabled = true;
+		return false;
+	}
 	
 	opl_init();
 	
-	SDL_PauseAudio(0); // unpause
+	SDL_PauseAudioDevice(audio_device, 0); // unpause
 	
 	return true;
 }
 
-void audio_cb( void *user_data, unsigned char *sdl_buffer, int howmuch )
+void audio_cb( void *user_data, unsigned char *sdl_buffer, int bytes_needed )
 {
 	(void)user_data;
 	
-	// prepare for conversion
-	howmuch /= audio_cvt.len_mult;
-	audio_cvt.buf = sdl_buffer;
-	audio_cvt.len = howmuch;
-	
 	static long ct = 0;
 	
-	SAMPLE_TYPE *feedme = (SAMPLE_TYPE *)sdl_buffer;
+	SAMPLE_TYPE *const feedme = (SAMPLE_TYPE *)sdl_buffer;
+	const int samples_needed = bytes_needed / BYTES_PER_SAMPLE;
 
 	if (!music_disabled && !music_stopped)
 	{
 		/* SYN: Simulate the fm synth chip */
 		SAMPLE_TYPE *music_pos = feedme;
-		long remaining = howmuch / BYTES_PER_SAMPLE;
+		long remaining = samples_needed;
 		while (remaining > 0)
 		{
 			while (ct < 0)
 			{
-				ct += freq;
+				ct += SAMPLE_RATE;
 				lds_update(); /* SYN: Do I need to use the return value for anything here? */
 			}
 			/* SYN: Okay, about the calculations below. I still don't 100% get what's going on, but...
-			- freq is samples/time as output by SDL.
+			- SAMPLE_RATE is samples/time as output by SDL.
 			- REFRESH is how often the play proc would have been called in Tyrian. Standard speed is
 			70Hz, which is the default value of 70.0f
 			- ct represents the margin between play time (representing # of samples) and tick speed of
@@ -126,12 +127,18 @@ void audio_cb( void *user_data, unsigned char *sdl_buffer, int howmuch )
 		}
 		
 		/* Reduce the music volume. */
-		int qu = howmuch / BYTES_PER_SAMPLE;
-		for (int smp = 0; smp < qu; smp++)
+		for (int smp = 0; smp < samples_needed; smp++)
 		{
 			feedme[smp] *= music_volume;
 		}
 	}
+	else
+	{
+		for (int smp = 0; smp < samples_needed; smp++)
+		{
+			feedme[smp] = 0;
+		}
+	}
 	
 	if (!samples_disabled)
 	{
@@ -141,7 +148,7 @@ void audio_cb( void *user_data, unsigned char *sdl_buffer, int howmuch )
 			float volume = sample_volume * (channel_vol[ch] / (float)SFX_CHANNELS);
 			
 			/* SYN: Don't copy more data than is in the channel! */
-			unsigned int qu = ((unsigned)howmuch > channel_len[ch] ? channel_len[ch] : (unsigned)howmuch) / BYTES_PER_SAMPLE;
+			const unsigned int qu = ((unsigned)bytes_needed > channel_len[ch] ? channel_len[ch] : (unsigned)bytes_needed) / BYTES_PER_SAMPLE;
 			for (unsigned int smp = 0; smp < qu; smp++)
 			{
 #if (BYTES_PER_SAMPLE == 2)
@@ -164,9 +171,6 @@ void audio_cb( void *user_data, unsigned char *sdl_buffer, int howmuch )
 			}
 		}
 	}
-	
-	// do conversion
-	SDL_ConvertAudio(&audio_cvt);
 }
 
 void deinit_audio( void )
@@ -174,9 +178,14 @@ void deinit_audio( void )
 	if (audio_disabled)
 		return;
 	
-	SDL_PauseAudio(1); // pause
+	if (audio_device != 0)
+	{
+		SDL_PauseAudioDevice(audio_device, 1); // pause
+		SDL_CloseAudioDevice(audio_device);
+		audio_device = 0;
+	}
 	
-	SDL_CloseAudio();
+	SDL_QuitSubSystem(SDL_INIT_AUDIO);
 	
 	for (unsigned int i = 0; i < SFX_CHANNELS; i++)
 	{
@@ -195,11 +204,12 @@ void load_music( void )
 	{
 		music_file = dir_fopen_die(data_dir(), "music.mus", "rb");
 		
-		efread(&song_count, sizeof(song_count), 1, music_file);
+		fread_u16_die(&song_count, 1, music_file);
 		
 		song_offset = malloc((song_count + 1) * sizeof(*song_offset));
 		
-		efread(song_offset, 4, song_count, music_file);
+		fread_u32_die(song_offset, song_count, music_file);
+
 		song_offset[song_count] = ftell_eof(music_file);
 	}
 }
@@ -209,7 +219,7 @@ void load_song( unsigned int song_num )
 	if (audio_disabled)
 		return;
 	
-	SDL_LockAudio();
+	SDL_LockAudioDevice(audio_device);
 	
 	if (song_num < song_count)
 	{
@@ -221,7 +231,7 @@ void load_song( unsigned int song_num )
 		fprintf(stderr, "warning: failed to load song %d\n", song_num + 1);
 	}
 	
-	SDL_UnlockAudio();
+	SDL_UnlockAudioDevice(audio_device);
 }
 
 void play_song( unsigned int song_num )
@@ -263,7 +273,7 @@ void JE_multiSamplePlay(JE_byte *buffer, JE_word size, JE_byte chan, JE_byte vol
 	if (audio_disabled || samples_disabled)
 		return;
 	
-	SDL_LockAudio();
+	SDL_LockAudioDevice(audio_device);
 	
 	free(channel_buffer[chan]);
 	
@@ -284,6 +294,6 @@ void JE_multiSamplePlay(JE_byte *buffer, JE_word size, JE_byte chan, JE_byte vol
 		}
 	}
 
-	SDL_UnlockAudio();
+	SDL_UnlockAudioDevice(audio_device);
 }
 
