@@ -23,17 +23,6 @@
 
 #define EVENT_MAXIMUM 2500
 
-#ifdef TYRIAN2000
-#define WEAP_NUM    818
-#define PORT_NUM    60
-#define ARMOR_NUM   4
-#define POWER_NUM   6
-#define ENGINE_NUM  6
-#define OPTION_NUM  37
-#define SHIP_NUM    18
-#define SHIELD_NUM  11
-#define SPECIAL_NUM 46
-#else
 #define WEAP_NUM    780
 #define PORT_NUM    42
 #define ARMOR_NUM   4
@@ -43,7 +32,6 @@
 #define SHIP_NUM    13
 #define SHIELD_NUM  10
 #define SPECIAL_NUM 46
-#endif
 
 #define ENEMY_NUM   850
 
