@@ -58,7 +58,6 @@ void adjust_difficulty( void );
 
 bool load_next_demo( void );
 bool replay_demo_keys( void );
-bool read_demo_keys( void );
 
 void JE_SFCodes( JE_byte playerNum_, JE_integer PX_, JE_integer PY_, JE_integer mouseX_, JE_integer mouseY_ );
 void JE_sort( void );
@@ -76,13 +75,13 @@ void JE_endLevelAni( void );
 void JE_drawCube( SDL_Surface * screen, JE_word x, JE_word y, JE_byte filter, JE_byte brightness );
 void JE_handleChat( void );
 bool str_pop_int( char *str, int *val );
-void JE_loadScreen( void );
+bool JE_loadScreen( void );
 void JE_operation( JE_byte slot );
 void JE_inGameDisplays( void );
 void JE_mainKeyboardInput( void );
 void JE_pauseGame( void );
 
-void JE_playerMovement( Player *this_player, JE_byte inputDevice, JE_byte playerNum, JE_word shipGr, Sprite2_array *shapes9ptr_, JE_word *mouseX, JE_word *mouseY );
+void JE_playerMovement( Player *this_player, JE_byte inputDevice, JE_byte playerNum, JE_word shipGr, Sprite2_array *shipGrPtr_, JE_word *mouseX, JE_word *mouseY );
 void JE_mainGamePlayerFunctions( void );
 const char *JE_getName( JE_byte pnum );
 
