@@ -23,20 +23,25 @@
 
 #include "SDL.h"
 
+enum
+{
+	MOUSE_POINTER_NORMAL = 0,
+	MOUSE_POINTER_UP,
+	MOUSE_POINTER_DOWN,
+	MOUSE_POINTER_LEFT,
+	MOUSE_POINTER_RIGHT,
+};
+
 extern bool has_mouse;
 extern bool mouse_has_three_buttons;
 
-extern JE_word lastMouseX, lastMouseY;
+extern bool mouseInactive;
 extern JE_byte mouseCursor;
 extern JE_word mouseX, mouseY, mouseButton;
 extern JE_word mouseXB, mouseYB;
 
-extern JE_byte mouseGrabShape[24 * 28];
-
-void JE_drawShapeTypeOne( JE_word x, JE_word y, JE_byte *shape );
-void JE_grabShapeTypeOne( JE_word x, JE_word y, JE_byte *shape );
-
 void JE_mouseStart( void );
+void JE_mouseStartFilter(Uint8 filter);
 void JE_mouseReplace( void );
 
 #endif /* MOUSE_H */
