@@ -16,12 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+#include "nortsong.h"
+
 #include "file.h"
 #include "joystick.h"
 #include "keyboard.h"
 #include "loudness.h"
 #include "musmast.h"
-#include "nortsong.h"
 #include "opentyr.h"
 #include "params.h"
 #include "sndmast.h"
@@ -101,7 +102,6 @@ void wait_delayorinput( JE_boolean keyboard, JE_boolean mouse, JE_boolean joysti
 void JE_loadSndFile( const char *effects_sndfile, const char *voices_sndfile )
 {
 	JE_byte y, z;
-	JE_word x;
 	JE_longint templ;
 	JE_longint sndPos[2][SAMPLE_COUNT + 1];
 	JE_word sndNum;
@@ -110,12 +110,11 @@ void JE_loadSndFile( const char *effects_sndfile, const char *voices_sndfile )
 	
 	/* SYN: Loading offsets into TYRIAN.SND */
 	fi = dir_fopen_die(data_dir(), effects_sndfile, "rb");
-	efread(&sndNum, sizeof(sndNum), 1, fi);
 
-	for (x = 0; x < sndNum; x++)
-	{
-		efread(&sndPos[0][x], sizeof(sndPos[0][x]), 1, fi);
-	}
+	fread_u16_die(&sndNum, 1, fi);
+
+	assert(sndNum < COUNTOF(sndPos[0]) - 1);
+	fread_s32_die(sndPos[0], sndNum, fi);
 	fseek(fi, 0, SEEK_END);
 	sndPos[0][sndNum] = ftell(fi); /* Store file size */
 
@@ -125,7 +124,7 @@ void JE_loadSndFile( const char *effects_sndfile, const char *voices_sndfile )
 		fxSize[z] = (sndPos[0][z+1] - sndPos[0][z]); /* Store sample sizes */
 		free(digiFx[z]);
 		digiFx[z] = malloc(fxSize[z]);
-		efread(digiFx[z], 1, fxSize[z], fi); /* JE: Load sample to buffer */
+		fread_u8_die(digiFx[z], fxSize[z], fi); /* JE: Load sample to buffer */
 	}
 
 	fclose(fi);
@@ -133,12 +132,10 @@ void JE_loadSndFile( const char *effects_sndfile, const char *voices_sndfile )
 	/* SYN: Loading offsets into VOICES.SND */
 	fi = dir_fopen_die(data_dir(), voices_sndfile, "rb");
 	
-	efread(&sndNum, sizeof(sndNum), 1, fi);
+	fread_u16_die(&sndNum, 1, fi);
 
-	for (x = 0; x < sndNum; x++)
-	{
-		efread(&sndPos[1][x], sizeof(sndPos[1][x]), 1, fi);
-	}
+	assert(sndNum < COUNTOF(sndPos[1]) - 1);
+	fread_s32_die(sndPos[1], sndNum, fi);
 	fseek(fi, 0, SEEK_END);
 	sndPos[1][sndNum] = ftell(fi); /* Store file size */
 
@@ -151,8 +148,9 @@ void JE_loadSndFile( const char *effects_sndfile, const char *voices_sndfile )
 		templ = (sndPos[1][y+1] - sndPos[1][y]) - 100; /* SYN: I'm not entirely sure what's going on here. */
 		if (templ < 1) templ = 1;
 		fxSize[z + y] = templ; /* Store sample sizes */
+		free(digiFx[z + y]);
 		digiFx[z + y] = malloc(fxSize[z + y]);
-		efread(digiFx[z + y], 1, fxSize[z + y], fi); /* JE: Load sample to buffer */
+		fread_u8_die(digiFx[z + y], fxSize[z + y], fi); /* JE: Load sample to buffer */
 	}
 
 	fclose(fi);
