@@ -16,21 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+#include "palette.h"
+
 #include "file.h"
 #include "nortsong.h"
 #include "opentyr.h"
-#include "palette.h"
 #include "video.h"
 
 #include <assert.h>
 
 static Uint32 rgb_to_yuv( int r, int g, int b );
 
-#ifdef TYRIAN2000
-#define PALETTE_COUNT 24
-#else
 #define PALETTE_COUNT 23
-#endif
 
 Palette palettes[PALETTE_COUNT];
 int palette_count;
@@ -51,9 +48,17 @@ void JE_loadPals( void )
 	{
 		for (int i = 0; i < 256; ++i)
 		{
-			palettes[p][i].r = getc(f) << 2;
-			palettes[p][i].g = getc(f) << 2;
-			palettes[p][i].b = getc(f) << 2;
+			// The VGA hardware palette used only 6 bits per component, so the values need to be rescaled to
+			// 8 bits. The naive way to do this is to simply do (c << 2), padding it with 0's, however this
+			// makes the maximum value 252 instead of the proper 255. A trick to fix this is to use the upper 2
+			// bits of the original value instead. This ensures that the value goes to 255 as the original goes
+			// to 63.
+
+			Uint8 rgb[3];
+			fread_u8_die(rgb, 3, f);
+			palettes[p][i].r = (rgb[0] << 2) | (rgb[0] >> 4);
+			palettes[p][i].g = (rgb[1] << 2) | (rgb[1] >> 4);
+			palettes[p][i].b = (rgb[2] << 2) | (rgb[2] >> 4);
 		}
 	}
 	
@@ -62,42 +67,22 @@ void JE_loadPals( void )
 
 void set_palette( Palette colors, unsigned int first_color, unsigned int last_color )
 {
-	SDL_Surface *const surface = SDL_GetVideoSurface();
-	const uint bpp = surface->format->BitsPerPixel;
-	
 	for (uint i = first_color; i <= last_color; ++i)
 	{
 		palette[i] = colors[i];
-		
-		if (bpp != 8)
-		{
-			rgb_palette[i] = SDL_MapRGB(surface->format, palette[i].r, palette[i].g, palette[i].b);
-			yuv_palette[i] = rgb_to_yuv(palette[i].r, palette[i].g, palette[i].b);
-		}
+		rgb_palette[i] = SDL_MapRGB(main_window_tex_format, palette[i].r, palette[i].g, palette[i].b);
+		yuv_palette[i] = rgb_to_yuv(palette[i].r, palette[i].g, palette[i].b);
 	}
-	
-	if (bpp == 8)
-		SDL_SetColors(surface, palette, first_color, last_color - first_color + 1);
 }
 
 void set_colors( SDL_Color color, unsigned int first_color, unsigned int last_color )
 {
-	SDL_Surface *const surface = SDL_GetVideoSurface();
-	const uint bpp = surface->format->BitsPerPixel;
-	
 	for (uint i = first_color; i <= last_color; ++i)
 	{
 		palette[i] = color;
-		
-		if (bpp != 8)
-		{
-			rgb_palette[i] = SDL_MapRGB(surface->format, palette[i].r, palette[i].g, palette[i].b);
-			yuv_palette[i] = rgb_to_yuv(palette[i].r, palette[i].g, palette[i].b);
-		}
+		rgb_palette[i] = SDL_MapRGB(main_window_tex_format, palette[i].r, palette[i].g, palette[i].b);
+		yuv_palette[i] = rgb_to_yuv(palette[i].r, palette[i].g, palette[i].b);
 	}
-	
-	if (bpp == 8)
-		SDL_SetColors(surface, palette, first_color, last_color - first_color + 1);
 }
 
 void init_step_fade_palette( int diff[256][3], Palette colors, unsigned int first_color, unsigned int last_color )
@@ -124,9 +109,6 @@ void step_fade_palette( int diff[256][3], int steps, unsigned int first_color, u
 {
 	assert(steps > 0);
 	
-	SDL_Surface *const surface = SDL_GetVideoSurface();
-	const uint bpp = surface->format->BitsPerPixel;
-	
 	for (unsigned int i = first_color; i <= last_color; i++)
 	{
 		int delta[3] = { diff[i][0] / steps, diff[i][1] / steps, diff[i][2] / steps };
@@ -139,15 +121,9 @@ void step_fade_palette( int diff[256][3], int steps, unsigned int first_color, u
 		palette[i].g += delta[1];
 		palette[i].b += delta[2];
 		
-		if (bpp != 8)
-		{
-			rgb_palette[i] = SDL_MapRGB(surface->format, palette[i].r, palette[i].g, palette[i].b);
-			yuv_palette[i] = rgb_to_yuv(palette[i].r, palette[i].g, palette[i].b);
-		}
+		rgb_palette[i] = SDL_MapRGB(main_window_tex_format, palette[i].r, palette[i].g, palette[i].b);
+		yuv_palette[i] = rgb_to_yuv(palette[i].r, palette[i].g, palette[i].b);
 	}
-	
-	if (bpp == 8)
-		SDL_SetColors(surface, palette, 0, 256);
 }
 
 
@@ -155,9 +131,6 @@ void fade_palette( Palette colors, int steps, unsigned int first_color, unsigned
 {
 	assert(steps > 0);
 	
-	SDL_Surface *const surface = SDL_GetVideoSurface();
-	const uint bpp = surface->format->BitsPerPixel;
-	
 	static int diff[256][3];
 	init_step_fade_palette(diff, colors, first_color, last_color);
 	
@@ -167,8 +140,7 @@ void fade_palette( Palette colors, int steps, unsigned int first_color, unsigned
 		
 		step_fade_palette(diff, steps, first_color, last_color);
 		
-		if (bpp != 8)
-			JE_showVGA();
+		JE_showVGA();
 		
 		wait_delay();
 	}
@@ -178,9 +150,6 @@ void fade_solid( SDL_Color color, int steps, unsigned int first_color, unsigned
 {
 	assert(steps > 0);
 	
-	SDL_Surface *const surface = SDL_GetVideoSurface();
-	const uint bpp = surface->format->BitsPerPixel;
-	
 	static int diff[256][3];
 	init_step_fade_solid(diff, color, first_color, last_color);
 	
@@ -190,8 +159,7 @@ void fade_solid( SDL_Color color, int steps, unsigned int first_color, unsigned
 		
 		step_fade_palette(diff, steps, first_color, last_color);
 		
-		if (bpp != 8)
-			JE_showVGA();
+		JE_showVGA();
 		
 		wait_delay();
 	}
