@@ -16,13 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+#include "params.h"
+
 #include "arg_parse.h"
 #include "file.h"
 #include "joystick.h"
 #include "loudness.h"
 #include "network.h"
 #include "opentyr.h"
-#include "params.h"
 #include "varz.h"
 #include "xmas.h"
 
@@ -138,7 +139,7 @@ void JE_paramCheck( int argc, char *argv[] )
 				}
 				
 				network_opponent_host = malloc(temp + 1);
-				strnztcpy(network_opponent_host, option.arg, temp);
+				SDL_strlcpy(network_opponent_host, option.arg, temp + 1);
 			}
 			else
 			{
