@@ -16,14 +16,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+#include "picload.h"
+
 #include "file.h"
 #include "opentyr.h"
 #include "palette.h"
 #include "pcxmast.h"
-#include "picload.h"
 #include "video.h"
 
 #include <string.h>
+#include <stdlib.h>
 
 void JE_loadPic(SDL_Surface *screen, JE_byte PCXnumber, JE_boolean storepal )
 {
@@ -37,12 +39,9 @@ void JE_loadPic(SDL_Surface *screen, JE_byte PCXnumber, JE_boolean storepal )
 		first = false;
 
 		Uint16 temp;
-		efread(&temp, sizeof(Uint16), 1, f);
-		for (int i = 0; i < PCX_NUM; i++)
-		{
-			efread(&pcxpos[i], sizeof(JE_longint), 1, f);
-		}
+		fread_u16_die(&temp, 1, f);
 
+		fread_s32_die(pcxpos, PCX_NUM, f);
 		pcxpos[PCX_NUM] = ftell_eof(f);
 	}
 
@@ -50,7 +49,7 @@ void JE_loadPic(SDL_Surface *screen, JE_byte PCXnumber, JE_boolean storepal )
 	Uint8 *buffer = malloc(size);
 
 	fseek(f, pcxpos[PCXnumber], SEEK_SET);
-	efread(buffer, sizeof(Uint8), size, f);
+	fread_u8_die(buffer, size, f);
 	fclose(f);
 
 	Uint8 *p = buffer;
@@ -65,7 +64,9 @@ void JE_loadPic(SDL_Surface *screen, JE_byte PCXnumber, JE_boolean storepal )
 			i += (*p & 0x3f);
 			memset(s, *(p + 1), (*p & 0x3f));
 			s += (*p & 0x3f); p += 2;
-		} else {
+		}
+		else
+		{
 			i++;
 			*s = *p;
 			s++; p++;
