@@ -20,7 +20,7 @@
 /*
  * This file is largely based on (and named after) a set of common reading/
  * writing functions used in Quake engines.  Its purpose is to allow extraction
- * of bytes, words, and dwords in a safe, endian adjused environment and should
+ * of bytes, words, and dwords in a safe, endian adjusted environment and should
  * probably be used in any situation where checking for buffer overflows
  * manually makes the code a godawful mess.
  *
@@ -34,12 +34,14 @@
  * and in normal Quake IIRC it is.  But our MO is to bail on failure, not
  * figure out what went wrong (making throws perfect).
  */
-
 #include "sizebuf.h"
-#include <assert.h>
 
 #include "SDL_endian.h"
 
+#include <assert.h>
+#include <stdio.h>
+#include <string.h>
+
 /* Construct buffer with the passed array and size */
 void SZ_Init(sizebuf_t * sz, Uint8 * buf, unsigned int size)
 {
