@@ -49,15 +49,17 @@ void JE_eventJump( JE_word jump );
 
 void JE_whoa( void );
 
-void JE_barX ( JE_word x1, JE_word y1, JE_word x2, JE_word y2, JE_byte col );
-
 Sint16 JE_newEnemy( int enemyOffset, Uint16 eDatI, Sint16 uniqueShapeTableI );
 void JE_drawEnemy( int enemyOffset );
 void JE_starShowVGA( void );
 
 void JE_main( void );
 void JE_loadMap( void );
-bool JE_titleScreen( JE_boolean animate );
+void networkStartScreen( void );
+bool titleScreen( void );
+bool newGame( void );
+bool newSuperArcadeGame( unsigned int i );
+void newSuperTyrianGame( void );
 void JE_readTextSync( void );
 void JE_displayText( void );
 
