@@ -16,6 +16,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+#include "varz.h"
+
 #include "config.h"
 #include "editship.h"
 #include "episodes.h"
@@ -31,7 +33,6 @@
 #include "opentyr.h"
 #include "shots.h"
 #include "sprite.h"
-#include "varz.h"
 #include "vga256d.h"
 #include "video.h"
 
@@ -215,7 +216,7 @@ bool play_demo = false, record_demo = false, stopped_demo = false;
 Uint8 demo_num = 0;
 FILE *demo_file = NULL;
 
-Uint8 demo_keys, next_demo_keys;
+Uint8 demo_keys;
 Uint16 demo_keys_wait;
 
 /* Sound Effects Queue */
@@ -257,7 +258,6 @@ JE_MultiEnemyType enemy;
 JE_EnemyAvailType enemyAvail;  /* values: 0: used, 1: free, 2: secret pick-up */
 JE_word enemyOffset;
 JE_word enemyOnScreen;
-JE_byte enemyShapeTables[6]; /* [1..6] */
 JE_word superEnemy254Jump;
 
 /*EnemyShotData*/
@@ -328,8 +328,8 @@ void JE_getShipInfo( void )
 {
 	JE_boolean extraShip, extraShip2;
 
-	shipGrPtr = &shapes9;
-	shipGr2ptr = &shapes9;
+	shipGrPtr = &spriteSheet9;
+	shipGr2ptr = &spriteSheet9;
 
 	powerAdd  = powerSys[player[0].items.generator].power;
 
@@ -452,7 +452,9 @@ void JE_tyrianHalt( JE_byte code )
 
 	free_main_shape_tables();
 
-	free_sprite2s(&shapes6);
+	free_sprite2s(&shopSpriteSheet);
+	free_sprite2s(&explosionSpriteSheet);
+	free_sprite2s(&destructSpriteSheet);
 
 	for (int i = 0; i < SAMPLE_COUNT; i++)
 	{
@@ -681,9 +683,9 @@ void JE_doSpecialShot( JE_byte playerNum, uint *armor, uint *shield )
 	if (player[0].items.special > 0)
 	{
 		if (shotRepeat[SHOT_SPECIAL] == 0 && specialWait == 0 && flareDuration < 2 && zinglonDuration < 2)
-			blit_sprite2(VGAScreen, 47, 4, shapes9, 94);
+			blit_sprite2(VGAScreen, 47, 4, spriteSheet9, 94);
 		else
-			blit_sprite2(VGAScreen, 47, 4, shapes9, 93);
+			blit_sprite2(VGAScreen, 47, 4, spriteSheet9, 93);
 	}
 
 	if (shotRepeat[SHOT_SPECIAL] > 0)
