@@ -16,14 +16,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+#include "vga256d.h"
+
 #include "config.h" // For fullscreen stuff
 #include "keyboard.h"
 #include "opentyr.h"
 #include "palette.h"
-#include "vga256d.h"
 #include "video.h"
 
 #include "SDL.h"
+
 #include <assert.h>
 #include <ctype.h>
 #include <math.h>
@@ -42,7 +44,7 @@ void JE_pix( SDL_Surface *surface, int x, int y, JE_byte c )
 
 void JE_pix3( SDL_Surface *surface, int x, int y, JE_byte c )
 {
-	/* Originally impemented as several direct accesses */
+	/* Originally implemented as several direct accesses */
 	JE_pix(surface, x, y, c);
 	JE_pix(surface, x - 1, y, c);
 	JE_pix(surface, x + 1, y, c);
@@ -150,10 +152,10 @@ void draw_segmented_gauge( SDL_Surface *surface, int x, int y, Uint8 color, uint
 
 	for (uint i = 0; i < segments; ++i)
 	{
-		fill_rectangle_hw(surface, x, y, segment_width, segment_height, color + 12);
+		fill_rectangle_wh(surface, x, y, segment_width, segment_height, color + 12);
 		x += segment_width + 1;
 	}
 	if (partial_segment > 0)
-		fill_rectangle_hw(surface, x, y, segment_width, segment_height, color + (12 * partial_segment / segment_value));
+		fill_rectangle_wh(surface, x, y, segment_width, segment_height, color + (12 * partial_segment / segment_value));
 }
 
