@@ -1,5 +1,5 @@
 /*
- * OpenTyrian Classic: A modern cross-platform port of Tyrian
+ * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
  * This program is free software; you can redistribute it and/or
@@ -19,19 +19,26 @@
 #ifndef VGA256D_H
 #define VGA256D_H
 
-#include "nortvars.h"
 #include "opentyr.h"
 
-extern JE_boolean mouseInstalled;
+#include "SDL.h"
 
-void JE_pix( JE_word x, JE_word y, JE_byte c );
-void JE_pix3( JE_word x, JE_word y, JE_byte c );
-void JE_rectangle( JE_word a, JE_word b, JE_word c, JE_word d, JE_word e );
-void JE_bar( JE_word a, JE_word b, JE_word c, JE_word d, JE_byte e );
-void JE_c_bar( JE_word a, JE_word b, JE_word c, JE_word d, JE_byte e );
-void JE_barShade( JE_word a, JE_word b, JE_word c, JE_word d );
-void JE_barBright( JE_word a, JE_word b, JE_word c, JE_word d );
+void JE_pix( SDL_Surface *surface, int x, int y, JE_byte c );
+void JE_pix3( SDL_Surface *surface, int x, int y, JE_byte c );
+void JE_rectangle( SDL_Surface *surface, int a, int b, int c, int d, int e );
+
+void fill_rectangle_xy( SDL_Surface *, int x, int y, int x2, int y2, Uint8 color );
+
+void JE_barShade( SDL_Surface *surface, int a, int b, int c, int d );
+void JE_barBright( SDL_Surface *surface, int a, int b, int c, int d );
+
+static inline void fill_rectangle_wh( SDL_Surface *surface, int x, int y, uint w, uint h, Uint8 color )
+{
+	SDL_Rect rect = { x, y, w, h };
+	SDL_FillRect(surface, &rect, color);
+}
+
+void draw_segmented_gauge( SDL_Surface *surface, int x, int y, Uint8 color, uint segment_width, uint segment_height, uint segment_value, uint value );
 
 #endif /* VGA256D_H */
 
-// kate: tab-width 4; vim: set noet:
