@@ -1,6 +1,6 @@
 /* 
- * OpenTyrian Classic: A modern cross-platform port of Tyrian
- * Copyright (C) 2007-2009  The OpenTyrian Development Team
+ * OpenTyrian: A modern cross-platform port of Tyrian
+ * Copyright (C) 2007-2010  The OpenTyrian Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,16 +23,20 @@
 
 #include "SDL.h"
 
-struct scaler_struct {
-	int scale;
-	void (*scaler16)(SDL_Surface *, SDL_Surface *, int);
-	void (*scaler32)(SDL_Surface *, SDL_Surface *, int);
+typedef void (*ScalerFunction)( SDL_Surface *src, SDL_Texture *dst );
+
+struct Scalers
+{
+	int width, height;
+	ScalerFunction scaler16, scaler32;
 	const char *name;
 };
 
-extern int scale, scaler;
-extern const struct scaler_struct scalers[9];
+extern uint scaler;
+extern const struct Scalers scalers[];
+extern const uint scalers_count;
+
+void set_scaler_by_name( const char *name );
 
 #endif /* VIDEO_SCALE_H */
 
-// kate: tab-width 4; vim: set noet:
